;;; -*- Mode:Lisp; Base:10; Package: http -*-

;;; File: html.lisp
;;; Program: 

;;; Author: Alberto Riva (alb@ipvaim.unipv.it)
;;; Created: Tue May  9, 1995
;;; Updated: Sun Nov  5, 1995


                                                                                ;Starting column for comments
;;; Contents:
;;;   Function    TEXT-HEADER 
;;;   Macro       DEFINE-PAGE 
;;;   Macro       WITH-OUTPUT-TO-HTTP 
;;;   Function    REGISTER-SYSTEM-PAGE 
;;;   Function    REGISTER-USER-PAGE 
;;;   Function    REGISTER-SYSTEM-FORM 
;;;   Function    REGISTER-USER-FORM 
;;;   Function    REGISTER-USER-GD 
;;;   Function    REGISTER-TOP-LEVEL-LINK 
;;;   Macro       HTML-OUT 
;;;   Function    HTML-BUILD 
;;;   Function    ANCHOR 
;;;   Function    REFERENCE 
;;;   Function    INLINE-IMAGE 
;;;   Function    IMAGE-MAP 
;;;   Function    INLINE-GD 
;;;   Macro       IN-FORM 
;;;   Function    SUBMIT-BUTTON 
;;;   Function    RESET-BUTTON 
;;;   Function    TEXT-FIELD 
;;;   Function    PASSWORD-FIELD 
;;;   Function    HIDDEN-FIELD 
;;;   Function    TEXT-AREA 
;;;   Function    MENU-SELECT 
;;;   Function    CHECKBOX 
;;;   Function    RADIO-BUTTON 
;;;   Function    BETWEEN 
;;;   Function    HEADER 
;;;   Function    BOLD 
;;;   Function    ITALIC 
;;;   Function    ADDRESS 
;;;   Function    BLOCKQUOTE 
;;;   Function    LISTING 
;;;   Function    PLAINTEXT 
;;;   Function    PREFORMATTED 
;;;   Function    CITATION 
;;;   Function    CODE 
;;;   Function    EMPHASIZED 
;;;   Function    FIXED 
;;;   Function    KEYBOARD 
;;;   Function    SAMPLE 
;;;   Function    STRONG 
;;;   Function    VARIABLE 
;;;   Function    STRIKE 
;;;   Function    CENTER 
;;;   Function    BLINK 
;;;   Function    SUPERSCRIPT 
;;;   Function    SUBSCRIPT 
;;;   Function    NOBREAK 
;;;   Function    FONT 
;;;   Function    COMMENT 
;;;   Function    HRULE 
;;;   Function    DEFINITION-LIST 
;;;   Function    NUMBERED-LIST 
;;;   Function    DOTTED-LIST 
;;;   Function    MENU 
;;;   Macro       IN-TABLE 
;;;   Function    TABLE-ROW 
;;;   Function    TABLE-HEADER 
;;;   Function    TABLE-CELL 
;;;   Function    CAPTION 
;;;   Function    LR-TREE 
;;;   Function    LR-TREE-AUX 
;;;   Function    TREE-SKELETON 
;;;   Function    TB-TREE 
;;;   Function    TB-TREE-AUX 
;;;   Function    HTML-PPRINT 
;;;   Macro       INCLUDE 
;;;   Function    COLOR-STRING 
;;;   Variable    *HALL-DEMO* 
;;; -*- -*-

(in-package :html)

(add-lispweb-service :hall)

;;; Objects

(defstruct page-style 
  code
  meta
  cookie
  auth
  background
  bgcolor
  text
  link
  vlink
  alink)

(defun define-page-style (&key code
			       meta
			       cookie
			       auth
			       background
			       bgcolor
			       text
			       link
			       vlink
			       alink)
  
  "Create a PAGE-STYLE structure, that can be passed to DEFINE-PAGE to specify
the default characteristics of the page."

  (make-page-style :code code
		   :meta meta
		   :cookie cookie
		   :auth auth
		   :background background
		   :bgcolor bgcolor
		   :text text
		   :link link
		   :vlink vlink
		   :alink alink))

(defun merge-styles (style1 style2)
  "Merge two PAGE-STYLE structures. When both structures specify a value for the
same attribute, the one from the first argument is used."
  
  (make-page-style :code (or (page-style-code style1) (page-style-code style2))
		   :meta (or (page-style-meta style1) (page-style-meta style2))
		   :cookie (or (page-style-cookie style1) (page-style-cookie style2))
		   :auth (or (page-style-auth style1) (page-style-auth style2))
		   :background (or (page-style-background style1) (page-style-background style2))
		   :bgcolor (or (page-style-bgcolor style1) (page-style-bgcolor style2))
		   :text (or (page-style-text style1) (page-style-text style2))
		   :link (or (page-style-link style1) (page-style-link style2))
		   :vlink (or (page-style-vlink style1) (page-style-vlink style2))
		   :alink (or (page-style-alink style1) (page-style-alink style2))))

;;; Table styles

(defstruct table-style
  border
  cellspacing
  cellpadding
  header-color
  cell-color
  header-background
  cell-background)

(defun define-table-style (&key border
				cellspacing
				cellpadding
				header-color
				cell-color
				header-background
				cell-background)
  
  "Create a TABLE-STYLE structure, that can be passed to IN-TABLE to specify
the default characteristics of the page."

  (make-page-style :code code
		   :meta meta
		   :cookie cookie
		   :auth auth
		   :background background
		   :bgcolor bgcolor
		   :text text
		   :link link
		   :vlink vlink
		   :alink alink))
	    
;;; Page definition

(defun source-page-button (function)
  (anchor (format nil "/view-source-page/~a" function)
	  (inline-image "marlowe.gif" :width 40 :height 30	:align :right)))

(defun marlowe-pprint (form &aux line (result ""))
  (with-input-from-string (str (write-to-string form :pretty t))
    (loop
     (unless (setq line (read-line str nil nil))
       (return))
     (setq result (format nil "~a~a~%" result line)))
    result))

;;; Page management

(defun register-top-level-link (anchor)
  "Define ANCHOR as a top-level link; it will appear in the default top page
of the server. Use it to make your applications easily accessible."

  (pushnew anchor http::*top-level-links* :test #'string=))

(defun page-class (page)
  (get page :lispweb-class))
   
(defun call-page (page &rest args)
  (declare (special http::lispweb-page-key))
  (when (http::keyed-class-p (page-class page))
    (push http::lispweb-page-key args))
  (apply (symbol-function page)
	 args))

;;; Utilities

(defun-hall html-build (&rest arguments)
  (apply #'format nil arguments))

(defun build-format-string-for-keywords (keywords)
  (format nil "~{~~:[~~; ~a=~~:*~~s~~]~}" keywords))

;;; HTML 3.0

(defun-hall anchor (url text &key target onclick onmouseover onmouseout url-is-string-p)
  "Format TEXT as an HTML anchor for URL."
  (if url-is-string-p
      (html-build "<A HREF=~a~:[~; TARGET=~:*~s~]~:[~; OnClick=~:*~s~]~:[~; OnMouseOver=~:*~s~]~:[~; OnMouseOut=~:*~s~]>~a</A>" url target onclick onmouseover onmouseout text)
      (html-build "<A HREF=~s~:[~; TARGET=~:*~s~]~:[~; OnClick=~:*~s~]~:[~; OnMouseOver=~:*~s~]~:[~; OnMouseOut=~:*~s~]>~a</A>" url target onclick onmouseover onmouseout text)))

(defun page-anchor (page text &key (full t) args target onclick onmouseover)
  "Build an HTML anchor for the specified PAGE, using TEXT as the
link content. Specify :full NIL only if the page is in the same
application and both the current and the target page takes no
arguments. If the page takes arguments, you can specify them
with :args '(arg1 arg2...)."
  (anchor (url page :full full :args args) text :target target 
	                                        :onclick onclick 
	                                        :onmouseover onmouseover))

(defun-hall reference (ref text &key target onclick onmouseover)
  "Format TEXT as an anchor for the reference REF."
  (html-build "<A NAME=~s~:[~; TARGET=~:*~a~]~:[~; OnClick=~:*~s~]~:[~; OnMouseOver=~:*~s~]>~a</A>" ref target  onclick onmouseover text))

(defun-hall inline-image (path &key align hspace vspace width height alt border usemap remote onabort onerror name)
  "Insert the inline image PATH. Use :remote T if the image
resides on a different server."
  (html-build "<IMG SRC=\"~:[/image/~;~]~a\"~:[~; ALIGN=~:*~a~]~:[~; BORDER=~:*~a~]~:[~; HSPACE=~:*~a~]~:[~; VSPACE=~:*~a~]~:[~; WIDTH=~:*~a~]~:[~; HEIGHT=~:*~a~]~:[~; ALT=~:*~s~]~:[~; USEMAP=~:*~s~]~:[~; OnAbort=~:*~s~]~:[~; OnError=~:*~s~]~:[~; NAME=~:*~s~]>" 
	      remote path align border hspace vspace width height alt usemap onabort onerror
              name))

(defun-hall image-map (map path &key target align hspace vspace width height alt border usemap)
  "Insert the inline image PATH and associate it with the indicated MAP."
  (html-build "<A HREF=\"/map/~a\"~:[~; TARGET=~:*~a~]><IMG SRC=\"~a\"~:[~; ALIGN=~:*~a~]~:[~; BORDER=~:*~a~]~:[~; HSPACE=~:*~a~]~:[~; VSPACE=~:*~a~]~:[~; WIDTH=~:*~a~]~:[~; HEIGHT=~:*~a~]~:[~; ALT=~:*~a~]~:[~; USEMAP=~:*~s~] ISMAP></A>~%" map target path align border hspace vspace width height alt usemap))

(defun-hall inline-gd (path &key align hspace vspace width height alt border)
  "Insert the gd-generated inline image PATH."
  (html-build "<IMG SRC=\"/gd/~a\"~:[~; ALIGN=~:*~a~]~:[~; BORDER=~:*~a~]~:[~; HSPACE=~:*~a~]~:[~; VSPACE=~:*~a~]~:[~; WIDTH=~:*~a~]~:[~; HEIGHT=~:*~a~]~:[~; ALT=~:*~a~]>"
	      path align border hspace vspace width height alt))

;;; Support for forms

(defmacro in-form ((&key action (method :post) name enctype onsubmit onreset) &body forms)
  "Create an HTML form. ACTION should be the URL of the funcion receiving the form
contents. METHOD can be :post or :get. The \"multipart/form-data\" ENCTYPE is not
yet supported."
  `(with-output-to-string (html-stream)
    (declare (special html-stream))
    (format html-stream 
	    "~&<FORM~:[~; ACTION=~:*~s~]~:[~; METHOD=~:*~a~]~:[~; NAME=~:*~a~]~:[~; ENCTYPE=~:*~a~]~:[~; OnSubmit=~:*~a~]~:[~; OnReset=~:*~a~] >~%" 
	    ,action
	    ,method
	    ,name
	    ,enctype			;Enctype=\"multipart/form-data\" 
	    ,onsubmit
	    ,onreset)
    ,@forms
    (format html-stream "</FORM>~%~%")))
		       
(defun-hall submit-button (value &key name onclick)
  "Create a submit button with the label VALUE."
  (html-build "<INPUT TYPE=\"submit\" VALUE=~s~:[~; NAME=~:*~s~]~:[~; OnClick=~:*~s~]>" value name onclick))

(defun-hall image-button (path &key name onclick)
  "Format image PATH as an image button."
  (html-build "<INPUT TYPE=\"image\" SRC=~s~:[~; NAME=~:*~s~]~:[~; OnClick=~:*~s~]>" path name onclick))

(defun-hall reset-button (value &key onclick)
  "Create a reset button with the label VALUE."
  (html-build "<INPUT TYPE=\"reset\" VALUE=~s~:[~; OnClick=~:*~s~]>" value onclick))

(defun-hall button (value &key onclick)
  "Create a button with the label VALUE."
  (html-build "<INPUT TYPE=\"button\" VALUE=~s~:[~; OnClick=~:*~s~]>" value onclick))
		    
(defun-hall text-field (label &key value size maxlength onblur onchange onfocus onselect)
  "Create a text field called LABEL."
  (html-build "<INPUT NAME=~s~:[~; VALUE=~:*~s~]~:[~; SIZE=~:*~s~]~:[~; MAXLENGTH=~:*~s~]~:[~; OnBlur=~:*~s~]~:[~; OnChange=~:*~s~]~:[~; OnFocus=~:*~s~]~:[~; OnSelect=~:*~s~]>" 
	      label value size maxlength onblur onchange onfocus onselect))

(defun-hall password-field (label &key value size maxlength onblur onchange onfocus onselect)
  "Create a password field called LABEL."
  (html-build "<INPUT TYPE=\"password\" NAME=~s~:[~; VALUE=~:*~s~]~:[~; SIZE=~:*~s~]~:[~; MAXLENGTH=~:*~s~]~:[~; OnBlur=~:*~s~]~:[~; OnChange=~:*~s~]~:[~; OnFocus=~:*~s~]~:[~; OnSelect=~:*~s~]>" 
	      label value size maxlength onblur onchange onfocus onselect))

(defun-hall hidden-field (label value)
  "Create a hidden field called LABEL with content VALUE."
  (html-build "<INPUT TYPE=\"hidden\" NAME=~s VALUE=~s>" label value))

(defun-hall text-area (label rows columns &key contents onblur onchange onfocus onselect)
  "Create a text area called LABEL with dimensions ROWSxCOLUMNS."
  (html-build "<TEXTAREA NAME=~s Rows=~d Cols=~d~:[~; OnBlur=~:*~s~]~:[~; OnChange=~:*~s~]~:[~; OnFocus=~:*~s~]~:[~; OnSelect=~:*~s~]>
~:[~;~:*~a~]
</TEXTAREA>" 
	      label rows columns  onblur onchange onfocus onselect contents))

(defun-hall filename-field (label &key value size maxlength)
  "Create a filename field called LABEL. (NOTE: requires the 
unimplemented \"multipart/form-data\ enctype."
  (html-build "<INPUT TYPE=\"file\" NAME=~s~:[~; VALUE=~:*~s~]~:[~; SIZE=~:*~s~]~:[~; MAXLENGTH=~:*~s~]>" label value size maxlength))

(defun-hall menu-select (label option-list &key value size multiple onblur onchange onfocus)
  "Create a menu called LABEL with choices OPTION-LIST."
  (let ((options (mapcan #'(lambda (item)
			     (list (equal item value) item))
			 option-list)))
    (html-build 
     "<SELECT NAME=~s~:[~; SIZE=~:*~s~]~:[~; MULTIPLE=~:*~s~]~:[~; OnBlur=~:*~s~]~:[~; OnChange=~:*~s~]~:[~; OnFocus=~:*~s~]>
~{<OPTION~:[~; SELECTED=\"T\"~]>~a~%~}</SELECT>" 
     label size multiple onblur onchange onfocus options)))

(defun-hall checkbox (label &key value checked onclick)
  "Create a checkbox called LABEL."
  (html-build "<INPUT TYPE=\"checkbox\" NAME=~s~:[~; VALUE=~:*~s~]~:[~; CHECKED=~:*~s~]~:[~; OnClick=~:*~s~]>" label value checked onclick))

(defun-hall radio-button (label &key value checked onclick)
  "Create a radio button called LABEL."
  (html-build "<INPUT TYPE=\"radio\" NAME=~s~:[~; VALUE=~:*~s~]~:[~; CHECKED=~:*~s~]~:[~; OnClick=~:*~s~]>" label value checked onclick))

;;; Typographical elements

(defun between (marker text)
  "General function to output a string between a pair of markers"
  (html-build "<~a>~a</~a>" marker text marker))

(defun-hall header (level text)
  "Format TEXT as a header of level LEVEL."
  (between (format nil "H~d" level) text))
  
(defun-hall bold (text)
  "Format TEXT as bold."
  (between "b" text))

(defun-hall italic (text)
  "Format TEXT as italic."
  (between "i" text))

(defun-hall address (text)
  "Format TEXT as an address."
  (between "ADDRESS" text))

(defun-hall blockquote (text)
  "Format TEXT as a quotation."
  (between "BLOCKQUOTE" text))

(defun-hall listing (text)
  "Format TEXT as a listing."
  (between "LISTING" text))

(defun-hall plaintext (text)
  "Format TEXT as plain text."
  (between "XMP" text))

(defun-hall preformatted (text)
  "Format TEXT as preformatted."
  (between "PRE" text))

(defun-hall citation (text)
  "Format TEXT as a citation."
  (between "CITE" text))

(defun-hall code (text)
  "Format TEXT as code."
  (between "CODE" text))

(defun-hall emphasized (text)
  "Format text as emphasized."
  (between "EM" text))

(defun-hall fixed (text)
  "Format TEXT using fixed-width font."
  (between "TT" text))

(defun-hall keyboard (text)
  "Format TEXT using keyboard font."
  (between "KBD" text))

(defun-hall sample (text)
  "Format TEXT as a code sample."
  (between "SAMPLE" text))

(defun-hall strong (text)
  "Format TEXT as strong."
  (between "STRONG" text))

(defun-hall var (text)
  "Format TEXT as a variable."
  (between "VAR" text))

(defun-hall strike (text)
  "Display TEXT as striked text."
  (between "STRIKE" text))

(defun-hall center (text)
  "Display TEXT centered on the page."
  (between "CENTER" text))

(defun-hall blink (text)
  "Display TEXT blinking."
  (between "BLINK" text))

(defun-hall superscript (text)
  "Display TEXT as a superscript."
  (between "SUP" text))

(defun-hall subscript (text)
  "Display TEXT as a subscript."
  (between "SUB" text))

(defun-hall nobreak (text)
  "Don't break TEXT."
  (between "NOBR" text))

(defun-hall font (text &key size color)
  "Display TEXT using a font of dimension SIZE and color COLOR."
  (html-build "<FONT~:[~; SIZE=~:*~s~]~:[~; COLOR=~:*~s~]>~a</FONT>" size color text))

(defun-hall comment (text)
  "Format TEXT as an HTML comment."
  (html-build "<!-- ~a -->" text))

(defun-hall hrule (&key size width align noshade)
  "Create a horizontal line."
  (html-build "<HR~:[~; SIZE=~:*~a~]~:[~; WIDTH=~:*~a~]~:[~; ALIGN=~:*~a~]~:[~; NOSHADE~]>" size width align noshade))

;;; Lists

(defun definition-list (pairs &optional list-header)
  "Build a list of definitions taken from PAIRS."
  (html-build "<DL>~% ~:[~;<LH>~:*~a</LH>~%~]~{~a~}</DL>" list-header
	  (mapcar #'(lambda (pair)
		      (format nil "<DT>~a~%<DD>~a~%" (car pair) (second pair)))
		  pairs)))

(defun-hall numbered-list (items &key list-header type start)
  "Build a numbered list of ITEMS."
  (html-build "<OL~:[~; TYPE=~:*~a~]~:[~; START=~:*~a~]>~%~:[~;<LH>~:*~a</LH>~%~]~{<LI>~a~%~}</OL>" type start list-header items))

(defun-hall dotted-list (items &key list-header type)
  "Build a dotted list of ITEMS."
  (html-build "<UL~:[~; TYPE=~:*~a~]>~%~:[~;<LH>~:*~a</LH>~%~]~{<LI>~a~%~}</UL>" type list-header items))

(defun-hall menu (items)
  "Build a menu of ITEMS."
  (html-build "<MENU>~%~{<LI>~a~%~}</MENU>" items))

;;; Tables

(defmacro in-table ((&key border cellspacing cellpadding width) &body forms)
  "Create an HTML table. Use TABLE-ROW in the FORMS to fill the table."
  `(with-output-to-string (html-stream)
     (declare (special html-stream))
     (format html-stream "~&<TABLE~:[ BORDER~; BORDER=~:*~d~]~:[~; CELLSPACING=~:*~d~]~:[~; CELLPADDING=~:*~d~]~:[~; WIDTH=~:*~d~]>~%" 
	     ,border ,cellspacing ,cellpadding ,width)
     ,@forms
     (format html-stream "</TABLE>~%~%")))

(defun-hall table-row (&rest items)	;align & valign not implemented yet.
  "Create a row of an HTML table. Use TABLE-HEADER and TABLE-CELL
to generate the ITEMS."
  (html-build "<TR>~%~{~a~}</TR>~%" items))

(defun-hall table-header (item &key rowspan colspan align valign width nowrap bgcolor)
  "Generate a header cell in an HTML table."
  (html-build "<TH~:[~; ROWSPAN=~:*~d~]~:[~; COLSPAN=~:*~d~]~:[~; ALIGN=~:*~a~]~:[~; VALIGN=~:*~a~]~:[~; NOWRAP~]~:[~; WIDTH=~:*~a~]~:[~; BGCOLOR=~:*~a~]>~a</TH>~%" 
	      rowspan colspan align valign nowrap width bgcolor item))

(defun-hall table-cell (item &key rowspan colspan align valign width nowrap bgcolor)
  "Generate a cell in an HTML table."
  (html-build "<TD~:[~; ROWSPAN=~:*~d~]~:[~; COLSPAN=~:*~d~]~:[~; ALIGN=~:*~a~]~:[~; VALIGN=~:*~a~]~:[~; NOWRAP~]~:[~; WIDTH=~:*~a~]~:[~; BGCOLOR=~:*~a~]>~a</TD>~%" 
	      rowspan colspan align valign nowrap width bgcolor item))

(defun-hall caption (text &key align)
  "Add a caption to an HTML table."
  (html-build "<CAPTION~:[~; ALIGN=~:*~s~]>~a</CAPTION>~%" align text))

;;; 

(defun-hall list-to-table (items &key (border 0) header align bold)
  "ITEMS is a list of lists; build a table putting each sublist in a different row."
  (let ((li (length (car items)))
	(la (length align))
	(lb (length bold)))
    (when (< la li)
      (setq align (append align (make-list (- li la) :initial-element :left))))
    (when (< lb li)
      (setq bold (append bold (make-list (- li lb)))))
    (in-table (:border border)
	      (when header
		(html-out (table-row (apply #'concatenate
					    'string
					    (mapcar #'table-header header)))))
	      (mapcar #'(lambda (it)
			  (html-out (table-row (apply #'concatenate 
						      'string 
						      (mapcar #'(lambda (i a b)
								  (if b
								      (table-header i :align a)
								    (table-cell i :align a)))
							      it align bold)))))
		      items))))

;;; Frames

(defun-hall frameset (&key rows cols frames border frameborder bordercolor)
  (html-build "<FRAMESET~:[~; ROWS=~:*~s~]~:[~; COLS=~:*~s~]~:[~; border=~:*~s~]~:[~; frameborder~:*~s~]~:[~; bordercolor~:*~s~]>~%~a</FRAMESET>~%" 
	      rows cols border frameborder bordercolor
	      (apply #'concatenate 'string (mapcar #'create-frames frames))))

(defun-hall frame (&key src name marginwidth marginheight scrolling noresize frameborder bordercolor)
  (html-build "<FRAME~:[~; src=~:*~s~]~:[~; name=~:*~s~]~:[~; marginwidth=~:*~s~]~:[~; marginheight=~:*~s~]~:[~; scrolling=~:*~s~]~:[~; noresize=~:*~s~]~:[~; frameborder=~:*~s~]~:[~; bordercolor=~:*~s~]>~%"
	      src name marginwidth marginheight scrolling noresize frameborder bordercolor))

;;same as frameset except that it calls frame2
(defun-hall frameset2 (&key rows cols frames border frameborder bordercolor)
  (html-build "<FRAMESET~:[~; ROWS=~:*~s~]~:[~; COLS=~:*~s~]~:[~; border=~:*~s~]~:[~; frameborder~:*~s~]~:[~; bordercolor~:*~s~]>~%~a</FRAMESET>~%" 
	      rows cols border frameborder bordercolor
	      (apply #'concatenate 'string (mapcar #'create-frames2 frames))))

;;same as frame except src uses ~a instead of ~s
(defun-hall frame2 (&key src name marginwidth marginheight scrolling noresize 
                         frameborder bordercolor)
  (html-build "<FRAME~:[~; src=~:*~a~]~:[~; name=~:*~s~]~:[~; marginwidth=~:*~s~]~:[~; marginheight=~:*~s~]~:[~; scrolling=~:*~s~]~:[~; noresize=~:*~s~]~:[~; frameborder=~:*~s~]~:[~; bordercolor=~:*~s~]>~%"
	      src name marginwidth marginheight scrolling noresize frameborder bordercolor))

(defun create-frames (frame-spec)
  (cond ((eq (car frame-spec) :frameset)
	 (apply #'frameset (cdr frame-spec)))
	(t (apply #'frame frame-spec))))
  
(defun create-frames2 (frame-spec)
  (cond ((eq (car frame-spec) :frameset)
	 (apply #'frameset2 (cdr frame-spec)))
	(t (apply #'frame2 frame-spec))))
  
;;; Java applets

(defun-hall applet (code &key codebase width height param-list)
  "Insert the Java applet CODE in the page."
  (with-output-to-string (applet-spec)
    (format applet-spec "<APPLET code=~s~:[~; CODEBASE=~:*~s~]~:[~; WIDTH=~:*~a~]~:[~; HEIGHT=~:*~a~]>~%" code codebase width height)
    (mapc #'(lambda (par)
	      (format applet-spec "<PARAM name=~s value=~s>~%" (car par) (second par)))
	  param-list)
    (format applet-spec "</APPLET>~%")))

;;; JavaScript

(defun-hall javascript (code &optional src)
  "Insert the JavaScript CODE in the page."
  (with-output-to-string (script)
    ;;(format script "<SCRIPT LANGUAGE=\"JavaScript\"~:[~; SRC=~:*~s~]>~%<!-- Begin JavaScript hiding~%" src)
    (format script "<SCRIPT LANGUAGE=\"JavaScript\"~:[~; SRC=~:*~s~]>~%" src)
    (format script "~a~%" code)
    ;;(format script "// End JavaScript hiding -->~%</SCRIPT>~%")
    (format script "~%</SCRIPT>~%")))

(defmacro noscript (&body forms)
  `(progn
     (html-out "<NOSCRIPT>~%")
     ,@forms
     (html-out "</NOSCRIPT>~%")))

;;; Client-side image maps

(defmacro client-side-map (name &body forms)
  "Create a client-side image mape called NAME. Use AREA in the
FORMS to generate the regions."
  `(with-output-to-string (html-stream)
     (declare (special html-stream))
     (format html-stream "<MAP NAME=~s>~%" ,name)
     ,@forms
     (format html-stream "</MAP>~%")))

(defun-hall area (coords &key shape href onmouseout onmouseover)
  "Declare an AREA in a client-side image map. COORDS should be 
a list of coordinates (e.g. (0 0 50 100) to describe a rectangle."
  (html-build "<AREA~:[~; SHAPE=~:*~a~] COORDS=~s~:[NOHREF~; HREF=~:*~s~]~:[~; OnMouseOver=~:*~s~]~:[~; OnMouseOut=~:*~s~]>~%" 
	      shape (coords-to-string coords) href onmouseover onmouseout))

(defun coords-to-string (&rest coords)
  (format nil "~{~d~^,~}" coords))

;;; Extensions

(defun lr-tree (tree)
  "Create a table to display tree in left-to-right layout. "
  (declare (special html-stream))
  (let ((skel (tree-skeleton tree)))
    (format html-stream "<TABLE BORDER>~%<TR>")
    (lr-tree-aux tree skel)
    (format html-stream "~%</TABLE>~%")))

(defun lr-tree-aux (tree skel)
  (cond ((null tree))
	((listp tree)
	 (format html-stream "<TD ROWSPAN=~d>~a" (car skel) (car tree))
	 (mapc #'lr-tree-aux
	       (cdr tree) (cdr skel)))
	(t (format html-stream "<TD>~a~%<TR>" tree))))

(defun tree-skeleton (tree &optional (depth 1))
  (let ((tail (mapcar #'(lambda (child)
			  (if (listp child)
			      (tree-skeleton child)
			      (list 1)))
		      (cdr tree))))
    (cons (reduce #'+ (mapcar #'car tail))
	  tail)))

(defun hierarchy (tree) 
  "Display a tree in hierarchical form, indenting the leaves under and at the right of the father."
  
  (format html-stream "<TABLE>~%")
  (mapc #'(lambda (root)
	    (if (listp root)
		(hierarchy-aux root 1)
	      (format html-stream "<TR VALIGN=TOP><TD>~a~%" root)))
	tree)
  (format html-stream "</TABLE>~%"))

(defun hierarchy-aux (nodes level)
  (let ((span (1- (count-nodes nodes))))
    (format html-stream "<TR VALIGN=TOP ROWSPAN=~d>" span)
    (dotimes (i level)
      (format html-stream "<TD>"))
    (format html-stream "~a~%" (car nodes))
    (incf level)
    (mapc #'(lambda (node)
	      (if (listp node)
		  (hierarchy-aux node level)
		(progn
		  (format html-stream "<TR VALIGN=TOP>")
		  (dotimes (i level)
		    (format html-stream "<TD>"))
		  (format html-stream "~a~%" node))))
	  (cdr nodes))))
  
(defun count-nodes (list)
  (let ((cnt 0))
    (mapc #'(lambda (n)
	      (incf cnt 
		    (if (listp n)
			(count-nodes n)
		      1)))
	  list)
    cnt))
	   
#|
(defun tb-tree (tree)
  "Create a table to display tree in top-to-bottom layout."
  (let ((skel (tree-skeleton tree)))
    (format html-stream "<TABLE BORDER>~%<TR>")
    (tb-tree-aux tree skel)
    (format html-stream "~%</TABLE>~%")))

(defun tb-tree-aux (tree skel)
  (cond ((null tree) nil)
	(t (print (car tree))
	   (print (mapcar #'car (cdr tree)))
	   

		 (mapcar #'car (cdr tree))
		 (mapcar #'tb-tree-aux (mapcar #'cdr (cdr tree))
			               (mapcar #'cdr (cdr skel))))))

|#

(defun html-pprint (form &aux line (result ""))
  "Pretty-print a lisp FORM to HTML."
  (with-input-from-string (str (write-to-string form :pretty t))
    (loop
     (unless (setq line (read-line str nil nil))
       (return))
     (setq result #{result line "<br>~%"}))  ;(format nil "~a~a<br>~%" result line)))
    result))

(defmacro include (filename)
  "Include the HTML file FILENAME in the page."
  `(with-output-to-string (html-stream)
    (declare (special html-stream))
    (let ((line ""))
      (with-open-file (input (lispweb-path ,filename))
	(loop 
	 (unless (setq line (read-line input nil nil))
	   (return))
	 (princ line html-stream)
	 (terpri html-stream))))))

(defun color-string (&key (red 0) (green 0) (blue 0))
  "Produce a string that encodes in RGB form the specified color. red,
  green and blue can be integers (in the range 0-255) or reals (in the
  range 0.0-1.0"
  
  (when (floatp red)
    (setq red (floor (* 255 red))))
  (when (floatp green)
    (setq green (floor (* 255 green))))
  (when (floatp blue)
    (setq blue (floor (* 255 blue))))
  (setq red (min 255 red)
	green (min 255 green)
	blue (min 255 blue))
  (setq red (max 0 red)
	green (max 0 green)
	blue (max 0 blue))
  (format nil "#~2,'0x~2,'0x~2,'0x" red green blue))

(defun format-http-info (&optional (lang :html))
  "Print the fields of the HTTP-INFO structure. LANG can be
:html (the default) or :text."
  (case lang
    (:html (html-out 
      (in-table (:border 0)
		(html-out (table-row (table-header "File descriptor:" :align :left)
				     (table-cell (http-info-fd *http-info*))))
		(html-out (table-row (table-header "Internet socket:" :align :left)
				     (table-cell (http-info-socket *http-info*))))
		(html-out (table-row (table-header "Internet address:" :align :left)
				     (table-cell (http-info-addr *http-info*))))
		(html-out (table-row (table-header "Request counter:" :align :left)
				     (table-cell (http-info-request-counter *http-info*))))
		(html-out (table-row (table-header "Method:" :align :left)
				     (table-cell (http-info-method *http-info*))))
		(html-out (table-row (table-header "Browser family:" :align :left)
				     (table-cell (http-info-browser *http-info*))))
		(html-out (table-row (table-header "Object:" :align :left)
				     (table-cell (http-info-request-object *http-info*))))
		(html-out (table-row (table-header "Arguments:" :align :left)
				     (table-cell (http::http-info-arguments *http-info*))))
		(html-out (table-row (table-header "Protocol:" :align :left)
				     (table-cell (http-info-protocol *http-info*))))
		(when (http-info-headers *http-info*)
		  (html-out (table-row (table-header "Headers:" :align :left)
				       (table-cell (http::print-header-field (car (http-info-headers *http-info*))))))
		  (do ((h (cddr (http-info-headers *http-info*)) (cddr h)))
		      ((null h))
		    (html-out (table-row (table-header "")
					 (table-cell (http::print-header-field (car h)))))))
		(when (http-info-fields *http-info*)
		  (html-out (table-row (table-header "Fields:" :align :left)
				       (table-cell (html-build "~:(~a~): ~a" (car (http-info-fields *http-info*)) 
							       (second (http-info-fields *http-info*))))))
		  (do ((h (cddr (http-info-fields *http-info*)) (cddr h)))
		      ((null h))
		    (html-out (table-row (table-header "")
					 (table-cell (html-build "~:(~a~): ~a" 
								 (car h)
								 (second h))))))))))
    (t (with-output-to-string (s)
	 (format s "File descriptor: ~s" (http-info-fd *http-info*))
	 (format s "Internet socket: ~s" (http-info-socket *http-info*))
	 
	 ))))

(defun refresh-meta (seconds)
  "Generate a META specification that will cause the page to be refreshed
after the specified number of seconds. "
  (list :http-equiv "Refresh" :content seconds))

(defun redirect-meta (seconds url)
  "Generate a META specification that will redirect the browser to
jump to a new url after the specified number of seconds."
  (list :http-equiv "Refresh" :content (format nil "~d; URL=~a" seconds url)))

;;; Definitions for on-line documentation

(setq *LispWeb-hall-demo* '(("Links")
			    (anchor (anchor "http://ipvaim.unipv.it:8080/" "LispWeb Server Top Page"))
			    (reference (reference "Top" "This is the top of the hill."))
			    ("Images")
			    (inline-image (inline-image "logo.gif" :align :middle))
			    (image-map (image-map "logo" "logo.gif"))
			    (inline-gd (inline-gd "prova"))
			    ("Forms")
			    ((in-form submit-button reset-button text-field text-area menu-select checkbox radio-button)
			     (in-form (:action (url 'lu::http-info))
			      (html-out "<P>Please fill in this form and press the ~a button.~%" (bold "Submit"))
			      (html-out "<P>Text field: ~a~%" (text-field "demo-text-field"))
			      (html-out "<P>Text area: ~a~%" (text-area "demo-text-area" 10 30))
			      (html-out "<P>Menu selection: ~a~%" (menu-select "demo-menu" '(1 2 4 8 16 32 64 128)))
			      (html-out "<P>Menu selection: ~a~%" (menu-select "demo-menu" '(1 2 4 8 16 32 64 128) :size 4))
			      (html-out "<P>Checkbox: ~a~%" (checkbox "demo-checkbox" :value "checked"))
			      (html-out "<P>Radio buttons: The Quick ~a Brown ~a Fox ~a~%" 
			       (radio-button "demo-radio" :value "Quick")
			       (radio-button "demo-radio" :value "Brown")
			       (radio-button "demo-radio" :value "Fox"))
			      (html-out "<P>~a" (submit-button "Submit this form"))
			      (html-out (reset-button "Reset this form"))))
			    ("Typographical elements")
			    (address (address "ariva@ipvaim.unipv.it"))
			    (blink (blink "This text is blinking"))
			    (blockquote (blockquote "A blockquoted paragraph."))
			    (bold (bold "This is bold."))
			    (center (center "This text is centered"))
			    (citation (citation "\"This is a citation\""))
			    (code (code "printf(\"Hello, world!\");"))
			    (comment (html-build "There is a ~a comment here!" (comment "hidden")))
			    (emphasized (emphasized "Emphasized text"))
			    (fixed (fixed "This is fixed text"))
			    (font (font "What is red and larger?" :size "+3" :color "#F00000"))
			    (header (header 2 "A level 2 header"))
			    (italic (italic "This is italic."))
			    (keyboard (keyboard "Keyboard? What's this??"))
			    (listing (listing "A listing"))
			    (plaintext (plaintext "This is plaintext"))
			    (preformatted (preformatted "  This  is 
preformatted 
    text"))
			    (sample (sample "This is a sample sample"))
			    (strike (strike "Strike!"))
			    (strong (strong "This is STRONG!"))
			    (subscript (subscript "test"))
			    (superscript (superscript "test"))
			    (var (var "z = x + y"))
			    ("Environments")
			    (definition-list (definition-list '(("Roger Waters" "Bass, vocals")
								("David Gilmour" "Guitar, vocals")
								("Rick Wright" "Keyboards, vocals")
								("Nick Mason" "Drums, vocals, Ferraris"))
						 "Pink Floyd:"))
			    (numbered-list (numbered-list '("Shine on you crazy diamond (part 1)"
							    "Welcome to the machine"
							    "Wish you were here"
							    "Have a cigar"
							    "Shine on you crazy diamond (part 2)")
					    :list-header "Wish You Were Here, 1976"))
			    (dotted-list (dotted-list '("Atom Hart Mother"
							"Meddle"
							"The Dark Side Of The Moon")
					  :list-header "Three Pink Floyd Albums"))
			    (menu (menu '("Wish You Were Here"
					  "Animals"
					  "The Wall")))
			    ((in-table table-row table-header table-cell) 
			     (in-table ()
			      (html-out (table-row (table-header "A Joint Probability Distribution" :colspan 4 :align :middle)))
			      (html-out (table-row (table-header "And") 
						   (table-header "not x" :align :middle) 
						   (table-header "x" :align :middle) 
						   (table-header "Total" :align :middle)))
			      (html-out (table-row (table-header "not y")
						   (table-cell "0.18" :align :right)
						   (table-cell "0.21" :align :right)
						   (table-cell "0.43" :align :right)))
			      (html-out (table-row (table-header "y")
						   (table-cell "0.37" :align :right)
						   (table-cell "0.24" :align :right)
						   (table-cell "0.57" :align :right)))
			      (html-out (table-row (table-header "Total")
						   (table-cell "0.55" :align :right)
						   (table-cell "0.45" :align :right)
						   (table-cell "1.00" :align :right)))
			      (html-out (caption "This is an example of a table."))))
			    ("Extensions")
			    (include (include "banner.html"))))
