(in-package :http)

#| 
Functions to manage the LispWeb processes. Processes are kept in the
*LispWeb-processes* list, and are indexed by either a keyword or a
number.
|#

(defun add-lispweb-process (key proc)
  (push (list key proc) *LispWeb-processes*)
  proc)

(defun get-lispweb-process (key) 
  "Return the process whose identifier is KEY. 
Returns NIL if the process was not found" 
  (second (assoc key *LispWeb-processes*)))

(defun remove-lispweb-process (key)
  (let ((proc (get-lispweb-process key)))
    (if proc 
	(setq *LispWeb-processes* 
	  (delete (assoc key *LispWeb-processes*) 
		  *LispWeb-processes*))
      (warn "Process not found"))))
  
(defun lispweb-process-kill (key)
  "Kill the process identified by KEY. Warns if
the process doesn't exist."
  (let ((proc (get-lispweb-process key)))
    (if proc 
	(progn
	  (mp:process-kill proc)
	  (remove-lispweb-process key))
      (warn "Process not found"))))

(defun lispweb-process-interrupt (key func)
  "Interrupt the process indentified by KEY with the function FUNC. 
Warns if the process doesn't exist."
  (let ((proc (get-lispweb-process key)))
    (if proc 
	(mp:process-interrupt proc func)
      (warn "Process not found"))))
      
(defun lispweb-process-info (key &optional (stream t))
  "Write to STREAM the information contained in the 
*HTTP-INFO* structure of the process identified by KEY."
  (lispweb-process-interrupt key 
			     #'(lambda ()
				 (pprint *http-info* stream))))

(defun lispweb-process-break (key)
  "Interrupt the process indentified by KEY and enter the debugger."
  (lispweb-process-interrupt key 
			     #'break))
