(in-package :tcp)

;;; Interface functions

(defun close-tcp-stream (stream)
  "Closes the stream's socket."
  (close stream))

(defun get-host-by-addr (address)
  (handler-case
      #+harlequin-unix-lisp
    (comm::get-host-name-by-address (comm::ip-address-string-to-number address))
      #+harlequin-pc-lisp
      "unknown-host" ;;for pc 
      #+allegro
      address
    (error (c)
      (declare (ignore c))
      address)))

#| for pc version (foreign:define-foreign-function 
 (crypt "_crypt")
 ((key :string) (salt :string))
 :result-type :string)
|#

;#-:lispworks(defun full-hostname ()
;  (let ((name (with-output-to-string (x)
;                (system::call-system-showing-output "hostname" 
;                                                    :prefix "" 
;                                                    :output-stream x 
;                                                    :show-cmd nil)))
;        (domain (with-output-to-string (x)
;                  (system::call-system-showing-output "domainname" 
;                                                      :prefix "" 
;                                                      :output-stream x 
;                                                      :show-cmd nil))))
;    (format nil "~a.~a" (subseq name 0 (1- (length name))) 
;                       (subseq domain 0 (1- (length domain))))))

#+:lispworks
(defun full-hostname ()
  (format nil "~a.~a" (system::machine-instance)
          *LispWeb-server-domain-name*))

#+allegro
(defun full-hostname ()
  (format nil "~a.~a" http::*machine-name* *LispWeb-server-domain-name*))
#|
(defun socket-fd (stream)
  "Return the FD associated with a socket stream"
  (excl::stream-input-fn stream))

;;; Support for external C functions

(load (merge-pathnames "hostadd.o" *load-pathname*)
      :foreign-files (list (merge-pathnames "libres.a" *load-pathname*))
      :unreferenced-lib-names '("_res_gethostbyaddr" "_crypt"))

(ff:defforeign 'res-hostadd 
    :entry-point "_res_hostadd" 
    :arguments '(string) 
    :return-type :integer)

(ff:defforeign 'res-hostname
    :entry-point "_res_hostname"
    :arguments '(string)
    :return-type :integer)

(defun get-host-by-name (name)
  (char*-to-string (res-hostname name)))

(ff::defforeign 'c-crypt
    :entry-point "_crypt"
    :arguments '(string string)
    :return-type :integer)

(defun crypt (key salt)
  (ff::char*-to-string (c-crypt key salt)))

(ff:defforeign 'c-full-hostname
    :entry-point "_full_hostname"
    :return-type :integer)

(defun full-hostname ()
  (ff:char*-to-string (c-full-hostname)))
|#

(defun get-host-by-name (name)
  name)

;(defun full-hostname ()
;  (system::getenv "HOST"))

;;; Two simple HTTP clients.

(defun send-mail (sender recipient text)
  "Send the mail message contained in text to recipient, using sender as the sender name."
  (let* ((host (subseq recipient (1+ (position #\@ recipient))))
	 (s (telnet host 25)))
    (when (= (read-from-string (read-line s)) 220)
      (format s "HELO~%")
      (force-output s)
      (format s "MAIL From:<~a>~%" sender)
      (force-output s)
      (format s "RCPT To:<~a>~%" recipient)
      (force-output s)
      (format s "DATA~%")
      (force-output s)
      (format s "~A~%.~%" text)
      (force-output s)
      (format s "QUIT~%")
      (force-output s))
    (close s)))

(defun lisp-http-server ()
  (let (inet-port)
    (setq http::*LispWeb-server-hostname* (full-hostname))
    (http::add-lispweb-process 
     :http
     (ipc::start-lisp-server-process 
      :server-name "HTTP Web Server"
      :startup-stream-process
      #'(lambda (stream &rest args) 
	  (declare (ignore args))
	  (http-server-startup-stream stream ipc:*inet-port-used*)) 
      :inet-port http::*http-port*
      :server-startup-announce-function 
      #'(lambda ()
	  (setf inet-port ipc:*inet-port-used*
		http::*http-port* ipc:*inet-port-used*
		(mp:process-name  mp:*current-process*)
		(format nil "Web Server port ~a" inet-port))
	  (format t "~&;Web server started on port ~a~%" inet-port))))
    (http::http-log-message "Lispweb server started on port ~a" http::*http-port*)))


(defun request-handler (stream)
  (format stream "Content-type: text/html~%~%<html><body><H1>test of allegro lisp web server</h1></body></hmtl>~%~%"))

;(defun http-server-startup-stream (stream porv)
;  (let ((client-address
;	 (getf (excl::stream-property-list stream)
;	       :pretty-socket-id)))
;    (mp:process-run-function 
;   (format nil "Web Client~@[ ~a~]"
;	   client-address)
;   #'request-handler
;   stream)))

(defun http-server-startup-stream (stream port)
  (let ((client-address
	 (getf (excl::stream-property-list stream)
	       :pretty-socket-id)))
    (let ((info (http::make-http-info :socket port
				      :stream stream
				      :fd nil
				      :addr client-address
				      :request-counter (incf http::*request-counter*))))
      (http::lw-log :debug ";;; Connection received from ~a~%" client-address)
      (let ((proc (mp:process-run-function (format nil "Http Handler #~d" http::*request-counter*)
                                           ;;nil
                                           'handle-request
                                           info)))
        (http::add-lispweb-process http::*request-counter* proc)
        (http::lw-log :debug ";;; Process ~d started~%" http::*request-counter*)))))

(defun handle-request (info)
  (let ((http::*http-info* info)
	(stream (http::http-info-stream info)))
    (unwind-protect
	(http::process-request http::*http-info*)
      #|
      (unless (equalp (http::header-value :connection) "Keep-Alive")
	(return))
      (unless (wait-for-input-available stream
					:whostate "Waiting for next request"
					:timeout 10)
	(return))
      (http::lw-log :debug ";;; Keeping connection open~%"))
      |#
      (progn
	(http::lw-log :debug ";;; CLosing connection~%")
	(close-tcp-stream stream) ;close stream at the end
	(http::remove-lispweb-process (http::http-info-request-counter http::*http-info*)))))) ; and remove process

(defun wait-for-input-available (stream &key whostate timeout)
  "Returns T if timedout, NIL if input available."
  (mp:process-wait-with-timeout whostate
                                timeout
                                #'listen
                                stream))

#+lispworks
(defun start-telnet-server ()
  (http::add-lispweb-process :telnet
    (comm::start-up-server :function 'listener-top-level;		           
			   :service http::*eval-port*
                           :process-name
			   (format nil "Telnet server port ~a" http::*eval-port*)))
  (http::http-log-message "Telnet server started on port ~a" http::*eval-port*))

#|
(defun start-telnet-server ()
  (let (inet-port)
    (http::add-lispweb-process :telnet
      (start-lisp-server-process 
       :server-name "Telnet Server"
       :inet-port http:*eval-port*
       :startup-stream-process #'(lambda (stm &rest args)
				   (declare (ignore args))
				   (let ((client-address
					  (getf (excl::stream-property-list stm) :pretty-socket-id)))
				     (format t "~&;Accepting telnet connection ~a/~a~%" client-address inet-port)
				     (mp:process-run-function
				      (format nil "Telnet Client~@[ ~a~]" client-address)
				      #'listener-top-level
				      stm)))
       :server-startup-announce-function #'(lambda ()
					     (setq inet-port *inet-port-used*)
					     (setf (mp:process-name mp:*current-process*)
					       (format nil "Telnet Server port ~a" inet-port))
					     (http::http-log-message "Telnet server started on port ~a" http:*eval-port*))))))
|#

(defvar *banner-string* "~a ~a~%Copyright The Harlequin Group Limited 1987--1995~%")


