;;; Mode: Lisp; Package: ocml

;;; Author: John Domingue

;;; The Open University

(in-package "OCML")

(in-ontology wsmo)

;;;goals and web services which implement IRS III

;;
(def-class mediate-input-role-values-goal (goal)
  "This goal transforms the inputs of a goal to the inputs of a web service."
  (;;(used-mediator :type mediator)
   ;;(has-post-condition :type post-condition)
   ;;(has-effect :type effect)
   ;;;;;extra to wsmo 
   (has-input-soap-binding :value (has-goal "sexpr")
                   :value (has-web-service "sexpr")
                   :value (has-mediation-ontology "sexpr")
                   :value (has-input-role-values "sexpr"))
   (has-output-soap-binding :value (has-mediated-input-role-values "sexpr"))
   (has-input-role :value has-goal 
                   :value has-web-service
                   :value has-input-role-values
                   :value has-mediation-ontology)
   (has-output-role :value has-mediated-input-role-values)
   (has-goal :type goal)
   (has-web-service :type web-service)
   (has-input-role-values :type list)
   (has-mediation-ontology :type ontology)
   (has-mediated-input-role-values :type list)
   (has-post-condition 
    :value 
    (kappa (goal) 
           (are-mediationally-equivalent
            (has-role-value goal has-input-role-values)
            (has-role-value goal has-mediated-input-role-values))))))

(def-class mediate-input-role-values-mediator (wg-mediator)
  ((has-source-component :value mediate-input-role-values-goal)))

(def-class mediate-input-role-values-web-service (web-service)
  (;;(used-mediator :value mediate-input-role-values-mediator)
   (has-internal-method :value irs-input-role-values-mediator)
   (has-capability :value mediate-input-role-values-capability)))

(def-class mediate-input-role-values-capability (capability)
  (   ;;(has-pre-condition :type pre-condition)
   ;;(has-post-condition :type post-condition)
   ;;(has-assumption :type assumption)
   ;;(has-effect :type effect)
   ;;extra to wsmo - directly links a capability to a goal
   ;;(solves-goal :value mediate-input-role-values-goal)
   (used-mediator :value mediate-input-role-values-mediator)
   ))

(def-class find-web-services-for-goal (goal)
  "This goal finds the web services which can solve a goal"
  ((has-input-role :value has-goal
                   :value has-ontology)
   (has-input-soap-binding :value (has-goal "sexpr")
                           :value (has-ontology "sexpr"))
   (has-output-role :value associated-web-services)
   (associated-web-services :type list)
   (has-output-soap-binding :value (associated-web-services "sexpr"))
   (has-goal :type goal)
   (has-ontology :type ontology)
   (has-post-condition 
    :value 
    (kappa (goal) 
           (and (member web-service (has-role-value goal associated-web-services))
                (can-solve-goal (has-role-value goal has-goal) web-service))))))

(def-class find-web-services-for-goal-mediator (wg-mediator)
  ((has-source-component :value find-web-services-for-goal)))

(def-class find-web-services-for-goal-web-service (web-service)
  (;;(used-mediator :value find-web-services-for-goal-mediator)
   (has-internal-method :value web-services-which-solve-goal-internal-method)
   (has-capability :value find-web-services-for-goal-web-service-capability)))

(def-class find-web-services-for-goal-web-service-capability (capability)
  (;;(solves-goal :value find-web-services-for-goal)
   (used-mediator :value find-web-services-for-goal-mediator)
   ))

(def-class suitable-web-service-goal (goal)
  "This goal checks if a web service is suitable for a goal by checking its 
applicability condition"
  ((has-input-role :value has-goal
                   :value has-actual-role-pairs
                   :value has-web-service
                   :value has-combined-oo-mediator-ontology)
   (has-input-soap-binding :value (has-goal "sexpr")
                           :value (has-actual-role-pairs "sexpr")
                           :value (has-web-service "sexpr")
                           :value (has-combined-oo-mediator-ontology "sexpr"))
   (has-output-role :value has-goal-and-web-service-instances)
   (has-goal-and-web-service-instances :type list)
   (has-output-soap-binding :value (has-goal-and-web-service-instances "sexpr"))
   (has-goal :type goal)
   (has-actual-role-pairs :type list)
   (has-web-service :type web-service)
   (has-combined-oo-mediator-ontology :type ontology)
   (has-post-condition 
    :value 
    (kappa (goal) 
           (is-suitable-for-goal (instantiate (has-role-value goal has-goal) 
                                              (has-role-value goal has-actual-role-pairs))
                                 web-service)))))

(def-class suitable-web-service-mediator (wg-mediator)
  ((has-source-component :value suitable-web-service-goal)))

(def-class suitable-web-service-web-service (web-service)
  (;;(used-mediator :value suitable-web-service-mediator)
   (has-internal-method :value suitable-web-service-internal-method)
   (has-capability :value suitable-web-service-capability)))

(def-class suitable-web-service-capability (capability)
  (;;(solves-goal :value suitable-web-service-goal)
   (used-mediator :value suitable-web-service-mediator)))