;;; Mode: Lisp; Package: ocml

;;; File created in WebOnto

(in-package "OCML")

(in-ontology aktive-portal-services)

(def-rule project-details-rule
    ((project-details ?project
                      (?leading-organization ?output ?member-organization 
                                             ?goals ?project-leader 
                                             ?project-member ?funding-source
                                             ?web-address ?area-of-interest))
     if
     (= ?leading-organization 
        (setofall ?x (has-leading-organization ?project ?x)))
     (= ?output 
        (setofall ?x (produces-output ?project ?x)))
     (= ?member-organization
        (setofall ?x (involves-organization ?project ?x)))
     (= ?goals
        (setofall ?x (has-goals ?project ?x)))
     (= ?project-leader
        (setofall ?x (has-project-leader ?project ?x)))
     (= ?project-member
        (setofall ?x (has-project-member ?project ?x)))
     (= ?funding-source
        (setofall ?x (funding-source ?project ?x)))
     (= ?web-address
        (setofall ?x (has-web-address ?project ?x)))
     (= ?area-of-interest
        (setofall ?x (addresses-generic-area-of-interest ?project ?x)))))

(def-rule project-research-areas-rule
    ((project-research-areas ?project ?research-area)
     if 
     (project ?project)
     (addresses-generic-area-of-interest ?project ?research-area)))

(def-rule project-publications-rule
    ((project-publications ?project ?publication)
     if 
     (project ?project)
     (produces-output ?project ?publication)
     (publication ?publication)))


(def-rule project-technologies-rule
    ((project-technologies ?project ?technology)
     if 
     (project ?project)
     (produces-output ?project ?technology)
     (technology ?technology)))

;;(progn (select-ontology 'akt-planet-stories-kb) (findany '?x '(shares-research-areas-with ibrow ?project ?x)))
(def-rule project-shares-research-areas-with-rule
    ((shares-research-areas-with ?project (?project2 ?project2-url
                                                     ?shared-areas))
     if 
     (= ?all-project-areas 
        (setofall ?x (addresses-generic-area-of-interest ?project ?x)))
     (project ?project2)
     (= ?all-project2-areas 
        (setofall ?y (addresses-generic-area-of-interest ?project2 ?y)))
     (not (== ?project ?project2))
     (= ?shared-areas (intersection ?all-project-areas ?all-project2-areas))
    (= ?l (length ?shared-areas))
    (> ?l 0)
    (has-web-address ?project2 ?project2-url)))

(def-rule project-interesting-to-rule
    ((interesting-to ?project (?person-full-name ?shared-areas))
     if 
     (= ?all-project-research-areas 
        (setofall ?x (addresses-generic-area-of-interest ?project ?x)))
     (person ?person)
     (not (has-project-member ?project ?person))
     (not (has-project-leader ?project ?person))
     (= ?all-person-research-areas 
        (setofall ?y (has-research-interest ?person ?y)))
     (= ?shared-areas (intersection ?all-project-research-areas ?all-person-research-areas))
     (= ?l (length ?shared-areas))
     (> ?l 0)
     (full-name ?person ?person-full-name)))


(def-rule aktive-project-members-rule
    ((project-members ?project ?name ?member-url)  
     if 
     (project ?project)
     (has-project-member ?project ?member)
     (full-name ?member ?name)
     (has-web-address ?member ?member-url))
    ((project-members ?project ?name ?member-url)  
     if 
     (project ?project)
     (has-project-leader ?project ?member)
     (full-name ?member ?name)     
     (has-web-address ?member ?member-url))
    ((project-members ?project ?name nil)  
     if 
     (project ?project)
     (has-project-member ?project ?member)
     (full-name ?member ?name))
    ((project-members ?project ?name nil)  
     if 
     (project ?project)
     (has-project-leader ?project ?member)
     (full-name ?member ?name)))

(def-rule aktive-project-web-page-rule
    ((project-web-page project ?project ?url)  
     if 
     (project ?project)
     (has-web-address ?project ?url)))
