;;; Mode: Lisp; Package: ocml

;;; File created in WebOnto

(in-package "OCML")

(in-ontology e-government-psms)




(def-class notify-service-provider-change-of-address-psm
           (primitive-method)
           ?psm
           ())


(def-class national-government-organization-notify-change-of-address-psm
           (notify-service-provider-change-of-address-psm)
           ?psm
           ()
           :own-slots
           ((tackles-task-type notify-change-of-address-task)
            (applicability-condition 
               (kappa (?psm)
                      (and (unit-of-organization (role-value ?psm 'has-target-provider)
                                       ?agency)
                           (national-government-organization ?agency)
                           (not (vulnerable-client (role-value ?psm 'has-client-id))))))))

(def-class automatically-calculate-who-to-notify-change-of-address-psm
           (notify-service-provider-change-of-address-psm)
           ?psm
           ()
           :own-slots
           ((tackles-task-type notify-change-of-address-task)
            (applicability-condition 
               (kappa (?psm)
                      (= (role-value ?psm 'has-target-provider)
                         'calculate-automatically)))))

(def-class county-level-organization-notify-change-of-address-psm
           (notify-service-provider-change-of-address-psm)
           ?psm
           ()
           :own-slots
           ((tackles-task-type notify-change-of-address-task)
            (applicability-condition 
               (kappa (?psm)
                      (and (unit-of-organization (role-value ?psm 'has-target-provider)
                                       ?agency)
                           (county-council-organization ?agency)
                           (not (vulnerable-client (role-value ?psm 'has-client-id))))))))

(def-class vulnerable-citizen-change-of-address-psm
           (notify-service-provider-change-of-address-psm)
           ?psm
           ()
           :own-slots
           ((tackles-task-type notify-change-of-address-task)
            (applicability-condition 
               (kappa (?psm)
                      (vulnerable-client (role-value ?psm 'has-client-id))))))

(def-relation vulnerable-client (?client-id)
  :sufficient
  (and (has-swift-id ?client ?client-id)
       (has-marital-status ?client  divorced-marital-status)
       (has-gender ?client female-gender)
       (has-ethnicity ?client ?ethnicity)
       (british-minority-ethnic-group-type ?ethnicity)))

(def-relation vulnerable-client2 (?client-id ?client)
  :sufficient
  (and (has-swift-id ?client ?client-id)
       (has-marital-status ?client  divorced-marital-status)
       (has-gender ?client female-gender)
       (has-ethnicity ?client ?ethnicity)
       (british-minority-ethnic-group-type ?ethnicity)))

#|



(def-class county-council-provider-notify-change-of-address-psm
           (notify-service-provider-change-of-address-psm)
           ?psm
           ()
           :own-slots
           ((tackles-task-type notify-change-of-address-task)
            (applicability-condition 
               (kappa (?psm)
                      (and (unit-of-organization (role-value ?psm 'has-target-provider)
                                       ?agency)
                           (county-council-organization ?agency))))))


(def-class department-of-work-and-pensions-notify-change-of-address-psm
           (notify-service-provider-change-of-address-psm)
           ?psm
           ()
           :own-slots
           ((tackles-task-type notify-change-of-address-task)
            (applicability-condition 
               (kappa (?psm)
                      (and (unit-of-organization (role-value ?psm 'has-target-provider)
                                       ?agency)
                           (= 'department-of-work-and-pensions ?agency))))))


(def-class health-care-service-provider-notify-change-of-address-psm
           (notify-service-provider-change-of-address-psm)
           ?psm
           ()
           :own-slots
           ((tackles-task-type notify-change-of-address-task)
            (applicability-condition 
               (kappa (?psm)
                      (and (unit-of-organization (role-value ?psm 'has-target-provider)
                                       ?agency)
                           (health-care-organization ?agency))))))


(def-class sme-provider-notify-change-of-address-psm
           (notify-service-provider-change-of-address-psm)
           ?psm
           ()
           :own-slots
           ((tackles-task-type notify-change-of-address-task)
            (applicability-condition 
               (kappa (?psm)
                      (and (unit-of-organization (role-value ?psm 'has-target-provider)
                                       ?agency)
                           (small-or-medium-sized-organization ?agency))))))
|#