;;; -*- Mode: LISP; Syntax: Common-lisp; Base: 10; Package: OCML;   -*-



(in-package "OCML")

(in-ontology gen-design-ocml)

(def-class PROBLEM-SOLVING-METHOD-FOR-PARAMETRIC-DESIGN 
  (problem-solving-method)
  :own-slots ((tackles-task-type  parametric-design)))

(def-class GEN-DESIGN-PSM 
  (problem-solving-method-for-parametric-design 
   decomposition-method)
  ((has-input-role :value has-design-operators)
   (has-output-role :value has-solution-state)
   (has-solution-state :type design-state)
   (has-design-operators :type design-operator)
   (has-output-mapping 
    :value '(lambda (?psm ?state)
              (the ?dm 
                (has-design-model ?state ?dm))))
   (has-body :value 
             '(lambda (?psm)
                (in-environment 
                 ((?s . (achieve-generic-subtask 
                         ?psm gen-design-control
                         has-current-pardes-task (the ?task 
                                                   (tackles-task 
                                                    ?psm ?task)))))
                 (if (design-state ?s)
                   ?s)))))
  :own-slots ((has-generic-subtasks '(gen-design-control))))


(def-class GEN-DESIGN-CONTROL (composite-task)
  ((has-input-role :value has-design-operators
                   :value has-current-pardes-task)
   (has-output-role :value has-solution-state)
   (has-solution-state :type design-state)
   (has-design-operators :type design-operator)
   (has-current-pardes-task :type parametric-design)
   (has-body :value 
             '(lambda (?psm)
                (in-environment 
                 ((?design-space . (achieve-generic-subtask 
                                    ?psm initialise-design-space 
                                    has-current-pardes-task 
                                    (role-value ?psm
                                                has-current-pardes-task))))
                 (REPEAT 
                  (in-environment 
                   ((?state . (achieve-generic-subtask 
                               ?psm Select-design-state 
                               has-design-space ?design-space)))
                   (if  (= ?state :nothing)
                     (RETURN :nothing)
                     (if (achieved (the-current-method) ?state)
                       (return ?state)
                       (do
                         (achieve-generic-subtask 
                          ?psm reflect-design-state
                          has-design-state ?state)
                         (achieve-generic-subtask 
                          ?psm design-from-state 
                          has-design-state ?state
                          has-design-space ?design-space))))))))))
  :own-slots ((has-generic-subtasks 
               '(initialise-design-space 
                 design-from-state 
                 reflect-design-state
                 select-design-state))))


(def-class SEARCH-CONTROL-RECORD ()
  "This structure records the control information associated
   with a state.  It is necessary to be able to support 
   generic control regimes"
  ((has-design-state :type design-state :cardinality 1)
   (has-design-focus :type design-focus  :cardinality 1)
   (has-current-operator :type design-operator :max-cardinality 1)
   (has-design-operators :type list  :cardinality 1)
   (has-design-foci :type list  :cardinality 1)))

(def-function THE-STATE-SEARCH-CONTROL-RECORD (?state)
  :body (the ?record (and (search-control-record ?record)
                          (has-design-state ?record ?state))))

(def-relation STATE-FULLY-EXPANDED (?state)
  :iff-def (and (= ?record (the-state-search-control-record ?state))
                (has-design-foci ?record nil)
                (has-design-operators ?record nil)))
                
                
(def-class DESIGN-SPACE () ?x
  "A design space is a set of design states associated 
   with a parametric design task"
  ((associated-with-task :type parametric-design :cardinality 1)
   (has-states :type set :cardinality 1 :default-value nil))
  :constraint (=> (member ?s (the ?set (has-states ?x ?set)))
                  (design-state ?s)))

(def-function DESIGN-SPACE-STATES (?space)
  :constraint (design-space ?space)
  :body (the ?states (has-states ?space ?states)))


(def-class INITIALISE-DESIGN-SPACE (composite-task) ?psm
  "Creates an initial design state (which is empty) and
   then returns a list containing only this state"
  ((has-input-role :value has-current-pardes-task)
   (has-output-role :value has-design-space)
   (has-control-role :value has-design-model)
   (has-current-pardes-task :type task)
   (has-design-space :type design-space)
   (has-body :value (lambda (?psm)
                      (in-environment 
                       ((?name . (new-symbol 'design-space)))
                       (tell (design-space 
                              ?name 
                              has-states nil
                              associated-with-task (role-value 
                                                    ?psm has-current-pardes-task)))
                       (achieve-generic-subtask ?psm
                                                 new-design-state  
                                                 has-design-model nil
                                                 has-design-space ?name)
                       ?name))))
  :own-slots ((has-generic-subtasks  '(new-design-state))))

(def-class NEW-DESIGN-STATE (composite-task) ?psm
  "Creates a design state"
  ((has-output-role :value has-design-state)
   (has-input-role  :value has-design-model
                    :value has-design-space)
   (has-design-space :type design-space)
   (has-design-state :type design-state)
   (has-design-model :type design-model)
   (has-body 
    :value (lambda (?psm)
             (in-environment 
              ((?design-model . (the ?dm2 
                                  (has-design-model 
                                   ?psm ?dm2)))
               (?design-space . (role-value 
                                 ?psm has-design-space))
               (?name . (new-symbol 'design-state)))
              (tell (design-state ?name
                                  has-design-model 
                                  ?design-model))
              (append-slot-value 
               ?design-space has-states  ?name)
              (achieve-generic-subtask ?psm 
                                       evaluate-design-state
                                       has-design-state ?name)
              ?name))))
  :own-slots ((has-generic-subtasks  '(evaluate-design-state))))



(def-class SELECT-DESIGN-STATE (goal-specification-task) ?task
  ((has-input-role :value has-design-space)
   (has-output-role :value has-design-state)
   (has-goal-expression 
    :value (kappa (?task)
                  (exists ?s
                          (and (design-state ?s)
                               (has-design-state ?task ?s)))))
   (has-design-space :type design-space)
   (has-design-state :type design-state)))

(def-function filter-feasible-consistent-states (?states)
  :body (setofall ?state 
                  (and (member ?state ?states)
                       (not (deadend-state ?state))
                       (not (constraint-violations 
                             ?state ?cs)))))

(def-function filter-maximal-states (?states)
  :body (setofall 
         ?state 
         (and (member ?state ?states)
              (has-design-model ?state ?dm)
              (= ?l (length ?dm))
              (not 
               (exists 
                ?state2
                (and (member ?state2 ?states)
                     (has-design-model ?state2 ?dm2)
                     (= ?l2 (length ?dm2))
                     (> ?l2 ?l)))))))

(def-function filter-cheapest-states (?states ?cost-order-rel)
  :body (setofall ?state 
                  (and (member ?state ?states)
                       (state-cost ?state ?cost)
                       (not (exists 
                             ?state2
                             (and (member ?state2 ?states)
                                  (state-cost ?state2 ?cost2)
                                  (holds ?cost-order-rel ?cost2 ?cost)))))))

(def-class CONSISTENT-MAX-CHEAPEST-STATE-SELECTION (primitive-method) 
  ((has-body 
    :value (lambda (?psm) 
             (in-environment 
              ((?cost-algebra . (role-value ?psm has-cost-algebra))
               (?cost-rel . (third ?cost-algebra))
               (?space . (role-value ?psm has-design-space))
               (?states . (design-space-states ?space)))
             (first
              (filter-cheapest-states
              (filter-maximal-states
               (filter-feasible-consistent-states ?states))
              ?cost-rel))))))
  :own-slots ((tackles-task-type  select-design-state)))

(def-class CONSISTENT-MAX-STATE-SELECTION (primitive-method) 
  ((has-body 
    :value (lambda (?psm) 
             (in-environment 
              ((?cost-algebra . (role-value ?psm has-cost-algebra))
               (?cost-rel . (third ?cost-algebra))
               (?space . (role-value ?psm has-design-space))
               (?states . (design-space-states ?space)))
             (first
              (filter-maximal-states
               (filter-feasible-consistent-states ?states)))))))
  :own-slots ((tackles-task-type  select-design-state)))

(def-class CONSISTENT-CHEAPEST-MAX-STATE-SELECTION (primitive-method) 
  ((has-body 
    :value (lambda (?psm) 
             (in-environment 
              ((?cost-algebra . (role-value ?psm has-cost-algebra))
               (?cost-rel . (third ?cost-algebra))
               (?space . (role-value ?psm has-design-space))
               (?states . (design-space-states ?space)))
             (first
              (filter-maximal-states
               (filter-cheapest-states
                (filter-feasible-consistent-states ?states)
                ?cost-rel)))))))
  :own-slots ((tackles-task-type  select-design-state)))



(def-class DESIGN-FROM-STATE (goal-specification-task) ?task
  "This task provides a place to define the main strategy
   to move from a state which is not a solution to one
   which is 'better'.  Of course, criteria are method-dependent.
   The :constraint option below states that the input state 
   is not a solution to the current problem"
  ((has-input-role :value has-design-state
                   :value has-design-space)
   (has-output-role :value has-output-state)
   (has-output-state :type design-state)
   (has-design-state :type design-state)
   (has-design-space :type design-space)
   (has-goal-expression 
    :value (kappa (?task ?s)
                  (design-state ?s))))                       
  :constraint (and (has-design-state ?task ?s)
                   (has-design-model ?s ?dm)
                   (= ?pd-problem (role-value 
                                   ?task has-current-pardes-task))
                   (not (achieved ?pd-problem ?dm))))

                        
                                       

(def-class EXTEND-INCOMPLETE-STATE (decomposition-method) 
  ((has-input-role :value has-design-state)
   (has-output-role :value generates-design-state)
   (has-design-state :type design-state)
   (generates-design-state :type design-state)
   (has-goal-expression 
    :value (kappa (?task ?s)
                  (design-model-extends 
                   (the ?dm (has-design-model ?s ?dm)) 
                   (the ?dm (has-design-model 
                             (role-value 
                              ?task has-design-state) 
                             ?dm)))))
   (has-body 
    :value 
    (lambda (?psm) 
      (in-environment 
       ((?state . (role-value ?psm has-design-state))
        (?design-model . (the ?dm (has-design-model 
                                   ?state ?dm)))
        (?constraints . (role-value ?psm has-constraints))
        (?parameters . (role-value ?psm has-parameters)))
       (if (deadend-state ?state)
         :nothing
         (if (constraint-violations ?state ?constraints)
           (tell (deadend-state ?state))
           (if (state-complete ?state ?parameters)
             (tell (solution-state ?state))
             (achieve-generic-subtask 
              ?psm
              generate-state-successor
              has-design-state ?state
              has-design-context :extend))))))))
  
  :own-slots ((tackles-task-type  design-from-state)
              (has-generic-subtasks generate-state-successor)))
                  



(def-relation deadend-state (?state)
 "True if a state is a 'failure'.  Failure is a 
   PSM-related concept.  For instance many psms
   work in a consistency-first style, and therefore
   regard inconsistent states as failure"
 :constraint (design-state ?state)
  )

;;(def-relation success-state (?state )
;;  "True if a state solves a design task"
;;  :constraint (design-state ?state))

;;(def-relation solution-state (?state)
;;  :iff-def (design-model-solution
;;            (the ?dm (has-design-model ?state ?dm))
;;            (the-current-task)))
                 

(def-relation state-complete (?state ?parameters)
  :iff-def (and (has-design-model ?state ?design-model)
                (design-model-complete ?design-model ?parameters)))

(def-relation constraint-violations (?state ?cs)
  "Associates a state to the constraints violated by 
   the design model associated with the state"
   :constraint (and (design-state ?state)
                    (list ?cs)
                    (every ?cs constraint)))
  
  

(def-class EVALUATE-DESIGN-STATE (composite-task) ?task
  ((has-input-role :value has-design-state)
   (has-design-state :type design-state)
   (has-body 
    :value (lambda (?task)
             (in-environment 
              ((?state . (role-value ?task has-design-state)))
              (achieve-generic-subtask ?task reflect-design-state
                                       has-design-state ?state)
              (achieve-generic-subtask ?task evaluate-consistency 
                                       has-design-state ?state)
              (achieve-generic-subtask ?task evaluate-completeness 
                                       has-design-state ?state)
              (achieve-generic-subtask ?task evaluate-cost 
                                       has-design-state ?state)
              (achieve-generic-subtask ?task evaluate-feasibility 
                                       has-design-state ?state)))))
  :own-slots ((has-generic-subtasks 
               '(evaluate-completeness 
                 evaluate-feasibility 
                 reflect-design-state
                  evaluate-cost 
                  evaluate-consistency ))))

(def-class REFLECT-DESIGN-STATE (goal-specification-task) ?task
  ((has-input-role :value has-design-state)
   (has-design-state :type design-state)))

(def-class EVALUATE-COMPLETENESS (primitive-task) ?task
  ((has-input-role :value has-design-state)
   (has-design-state :type design-state)
   (has-body 
    :value (lambda (?task) 
             (in-environment 
              ((?state . (role-value ?task has-design-state))
               (?design-model . (the ?dm (has-design-model ?state ?dm)))
               (?parameters . (role-value ?task has-parameters)))
              (if (design-model-complete ?design-model ?parameters)
                (tell (state-complete ?state))))))))

(def-class EVALUATE-COST (goal-specification-task) ?task
  ((has-input-role :value has-design-state)
   (has-output-role :value has-cost)
   (has-design-state :type design-state)
   (has-cost :type cost)
   (has-goal-expression 
    :value (kappa (?task ?cost)
                  (and (cost ?cost)
                       (has-cost ?task ?cost))))))

(def-class DEFAULT-COST-EVALUATION (primitive-method) ?psm
  ((has-body 
    :value (lambda (?psm)
             (in-environment 
              ((?state . (role-value ?psm has-design-state))
               (?design-model . (the ?dm (has-design-model 
                                          ?state ?dm)))
               (?cost-fun . (role-value ?psm has-cost-function))
               (?cost . (call ?cost-fun ?design-model)))
              (do
                (tell (state-cost ?state ?cost))
                ?cost)))))
  :own-slots ((tackles-task-type  evaluate-cost)))


(def-class EVALUATE-CONSISTENCY (primitive-task) ?task
  ((has-input-role :value has-design-state)
   (has-design-state :type design-state)
   (has-body 
    :value (lambda (?task)
             (in-environment 
              ((?state . (role-value ?task has-design-state))
               (?design-model . (the ?dm (has-design-model 
                                          ?state ?dm)))
               (?constraints . (role-value ?task has-constraints))
               (?vs . (setofall ?c (and (member ?c ?constraints)
                                        (design-model-violates 
                                         ?design-model ?c)))))
              (if (not (null ?vs))
                (tell (constraint-violations ?state ?vs)))
              ?vs)))))


(def-class EVALUATE-FEASIBILITY (primitive-task) ?task
  ((has-input-role :value has-design-state)
   (has-design-state :type design-state)
   (has-body :value (lambda (?task) 
                      true))))


 
;;(def-procedure design-succeeds (?state)
;;;  :body (RETURN
;;;         (do
 ;;;          (tell (success-state ?state))
 ;;;          ?s)))




(def-class generate-state-successor (composite-task)
  ((has-input-role :value has-design-state
                   :value has-design-context)
   (has-output-role :value generates-design-state)
   (has-design-context :type design-context)
   (has-design-state :type design-state)
   (generates-design-state :type design-state)
   (has-body :value (lambda (?task)
               (in-environment 
                ((?state . (role-value ?task has-design-state))
                 (?params . (role-value ?task has-parameters))
                 (?context . (role-value ?task has-design-context)))
                
                (if (search-control-record ?record
                                           has-design-state ?state)
                  
                  ;;we are effectively backtracking to ?state
                  (in-environment 
                   ((?result .  (achieve-generic-subtask 
                                 ?task resume-state
                                 has-design-state ?state
                                 has-design-context ?context)))
                   (if (design-state ?result)
                     ?result
                     (achieve-generic-subtask ?task 
                                              design-from-context
                                              has-design-state ?state
                                              has-design-context ?context)))
                  
                  ;;?state is a newly-created state
                  (in-environment 
                   ((?foci . (achieve-generic-subtask 
                              ?task collect-state-foci
                              has-design-state ?state
                              has-design-context ?context)))
                   (new-search-control-record ?state ?foci)
                   (achieve-generic-subtask 
                    ?task design-from-context
                     has-design-state ?state
                     has-design-context ?context)))))))
   :own-slots ((has-generic-subtasks '(resume-state 
                                       design-from-context
                                       collect-state-foci))))
  




(def-class collect-state-foci (goal-specification-task) ?task
  ((has-input-role :value  has-design-context
                   :value  has-design-state)
   (has-output-role :value  has-design-foci)
   (has-design-foci :type list)
   (has-design-state :type design-state)
   (has-design-context :type design-context)))

(def-class COLLECT-COMPUTABLE-PARAMETERS (primitive-method) 
  ((has-body 
    :value (lambda (?psm)
             (all-computable-parameters 
              (role-value ?psm has-parameters) 
              (the ?dm (has-design-model 
                        (role-value ?psm has-design-state) 
                        ?dm))))))
  :own-slots ((tackles-task-type collect-state-foci)
              (applicability-condition 
               (kappa (?task)
                      (= (role-value ?task 
                                     'has-design-context)
                         :extend)))))


(def-procedure  new-search-control-record (?state ?foci)
  :body (tell (search-control-record 
               (new-symbol 'state-search-control-record)
               has-design-state ?state
               has-design-foci ?foci)))

    

(def-class DESIGN-FROM-CONTEXT (composite-task) ?task
  ((has-input-role :value has-design-state
                   :value has-design-context)
   (has-output-role :value generates-design-state)
   (has-control-role :value has-design-foci
                     :value has-search-control-record)
   (has-design-context :type design-context)
   (has-design-state :type design-state)
   (generates-design-state :type design-state)
   (has-body 
    :value 
    (lambda (?task) 
      (REPEAT
       (in-environment 
        ((?state . (role-value ?task has-design-state))
         (?record . (the-state-search-control-record 
                     ?state))
         (?foci . (the-slot-value ?record 
                                  'has-design-foci))
         (?sub . (instantiate-generic-subtask 
                    ?task select-design-focus
                    has-design-foci ?foci))
         (?focus . (solve-task ?sub)))
        (if (achieved ?sub ?focus)
          (do
            (achieve-generic-subtask 
             ?task
             update-search-control-record-on-focus-selection 
             has-search-control-record ?record
             has-design-focus ?focus)
            (in-environment 
             ((?ops . (achieve-generic-subtask 
                       ?task collect-focus-operators
                       has-design-focus ?focus))
              (?sorted-ops . (achieve-generic-subtask 
                              ?task sort-design-operators
                              has-design-operators ?ops)))
             (if (null ?sorted-ops)
               (achieve-generic-subtask 
                ?task
                update-search-control-record-on-focus-failure 
                has-search-control-record ?record
                 has-design-focus ?focus)
               (do
                 (set-slot-value ?record 
                                 has-design-operators 
                                 ?sorted-ops)
                 (in-environment 
                  ((?result .  (achieve-generic-subtask 
                                ?task design-from-focus 
                                has-design-state ?state)))
                  (if (design-state ?result)
                    (return ?result)))))))
          (do
            (tell (deadend-state ?state))
            (return :nothing))))))))
  :own-slots ((has-generic-subtasks 
               '(select-design-focus 
                 collect-focus-operators 
                 sort-design-operators 
                 update-search-control-record-on-focus-failure 
                 update-search-control-record-on-focus-selection  
                 design-from-focus))))



(def-class UPDATE-SEARCH-CONTROL-RECORD-ON-FOCUS-FAILURE (goal-specification-task) ?task
  ((has-input-role :value  has-search-control-record
                   :value has-design-focus)
   (has-design-focus :type design-focus)
   (has-search-control-record :type search-control-record)))

(def-class DEFAULT-SEARCH-CONTROL-RECORD-ON-FOCUS-FAILURE-UPDATE (primitive-method) ?psm
  ((has-body 
    :value (lambda (?psm) :nothing)))
   :own-slots ((tackles-task-type update-search-control-record-on-focus-failure)))


(def-class update-search-control-record-on-focus-selection (goal-specification-task) 
  ((has-input-role :value has-search-control-record
                   :value has-design-focus)
   (has-design-focus :type design-focus)
   (has-search-control-record :type search-control-record)))

(def-class DEFAULT-search-control-record-on-focus-selection-UPDATE (primitive-method) ?psm
  ((has-body  
    :value (lambda (?psm)
             (in-environment 
              ((?focus . (role-value ?psm has-design-focus))
               (?record . (role-value ?psm has-search-control-record)))
              (set-slot-value ?record has-design-foci
                              (remove ?focus (the-slot-value 
                                              ?record has-design-foci)))
              (set-slot-value ?record has-design-focus ?focus)))))
     :own-slots ((tackles-task-type update-search-control-record-on-focus-selection)))

(def-class RESUME-STATE (goal-specification-task) 
  ((has-input-role :value has-design-state
                   :value has-design-context) 
   (has-output-role :value has-output-design-state)
   (has-design-state :type design-state)
   (has-design-context :type design-context)
   (has-output-design-state :type design-state)
   (has-goal :value (kappa (?task ?s)
                    (and (design-state ?s)
                         (not (= ?s (role-value 
                                     ?task 
                                     has-design-state))))))))


(def-class TRY-DIFFERENT-STATE-OPERATOR (primitive-method) ?psm
  ((has-body  
    :value (lambda (?psm)
             (achieve-generic-subtask 
              ?psm design-from-focus 
              has-design-state (role-value ?psm has-design-state)))))
  :own-slots ((tackles-task-type resume-state)
              (applicability-condition 
               (kappa 
                (?task9)
                (basic-operator
                 (the ?op 
                   (has-current-operator 
                    (the-state-search-control-record 
                     (role-value 
                      ?task9 has-design-state)) 
                    ?op)))))))

(def-class RETRY-STATE-OPERATOR (primitive-method) ?psm
  ((has-body  
    :value (lambda (?psm)
             (in-environment
              ((?state . (role-value ?psm has-design-state))
               (?record . (the-state-search-control-record ?state))
               (?op . (the ?op2 (has-current-operator ?record ?op2))))
              (if (has-design-focus ?record ?focus)
                (in-environment 
                 ((?sub . (instantiate-generic-subtask 
                           ?psm try-design-operator
                           has-design-operator ?op
                           has-design-focus ?focus
                           has-design-model (the-slot-value 
                                             ?state 
                                             'has-design-model)))
                  (?result . (solve-task ?sub)))
                 (if (achieved ?sub2 ?result)
                   ?result
                   (achieve-generic-subtask 
                    ?psm design-from-focus
                    has-design-state ?state))))))))
  :own-slots ((tackles-task-type resume-state)
              (applicability-condition 
               (kappa 
                (?task9)
                (multiple-operator
                 (the ?op 
                   (has-current-operator 
                    (the-state-search-control-record 
                     (role-value 
                      ?task9 has-design-state)) 
                    ?op)))))))





(def-class DESIGN-FROM-FOCUS (composite-task) 
  ((has-input-role :value has-design-state) 
   (has-output-role :value has-output-design-state)
   (has-control-role :value has-design-model
                     :value has-design-operator)
   (has-design-state :type design-state)
   (has-output-design-state :type design-state)
   (has-body 
    :value 
    (lambda (?task)
      (REPEAT
       (in-environment 
        ((?state . (role-value ?task has-design-state))
         (?record . (the-state-search-control-record 
                     ?state))
         (?focus . (the-slot-value 
                    ?record 'has-design-focus))
         (?ops .  (the-slot-value 
                   ?record 'has-design-operators))
         (?sub . (instantiate-generic-subtask 
                  ?task select-design-operator
                  has-design-focus ?focus
                  has-design-operators ?ops))
         (?op . (solve-task ?sub)))
        (set-slot-value ?record has-current-operator ?op)
        (if (achieved ?sub ?op)
          (DO
            (set-slot-value 
             ?record 
             has-design-operators 
             (remove ?op ?ops))
            (in-environment
             ((?sub2 . (instantiate-generic-subtask 
                        ?task try-design-operator
                        has-design-operator ?op
                        has-design-focus ?focus
                        has-design-model (the-slot-value 
                                          ?state 
                                          'has-design-model)))
              (?result . (solve-task ?sub2)))
             (if (achieved ?sub2 ?result)
               (RETURN ?result))))
          (RETURN :nothing)))))))
  :own-slots ((has-generic-subtasks '(select-design-operator
                                      try-design-operator))))


(def-class TRY-DESIGN-OPERATOR (goal-specification-task) ?task
  ((has-input-role :value has-design-operator
                   :value has-design-focus
                   :value has-design-model)
   (has-output-role :value generates-design-state)
   (has-design-focus :type design-focus)
   (has-design-operator :type design-operator)
   (has-design-model :type design-model)
   (generates-design-state :type design-state)
   (has-goal-expression 
    :value (kappa (?task8 ?s)
                  (and (design-state ?s)
                       (generates-design-state ?task8 ?s))))))


(def-class TRY-DESIGN-EXTENSION-OPERATOR (primitive-method) 
  ((has-body 
    :value 
    (lambda (?psm) 
      (in-environment 
       ((?dm . (role-value ?psm 'has-design-model))
        (?focus . (role-value ?psm 'has-design-focus))
        (?value . (apply-design-extension-operator
                   ?focus ?dm (role-value ?psm 'has-design-operator))))
       (if (not (= ?value :nothing))
         (achieve-generic-subtask 
          ?psm new-design-state
          has-design-model (cons 
                            (cons ?focus ?value)
                            ?dm)))))))
  :own-slots ((tackles-task-type try-design-operator)
              (applicability-condition 
               (kappa 
                (?task5)
                (design-extension-operator
                 (role-value 
                  ?task5 has-design-operator))))))
           


(def-function apply-design-extension-operator (?param ?dm ?op)
  :constraint (and (parameter ?param)
                   (design-model ?dm)
                   (design-extension-operator ?op))
  :body (call (the ?body 
                (has-body ?op ?body))
              ?param
              ?dm))
                      


(def-class SELECT-DESIGN-OPERATOR (goal-specification-task) ?task
  ((has-input-role :value has-design-operators
                   :value has-design-focus)
   (has-output-role :value has-selected-operator)
   (has-design-operators :type list)
   (has-selected-operator :type design-operator)
   (has-design-focus :type design-focus)
   (has-goal-expression 
    :value (kappa (?task ?op)
                  (and (design-operator ?op)
                       (has-selected-operator ?task ?op))))))


(def-class DEFAULT-OPERATOR-SELECTION (primitive-method) ?psm
  ((has-body 
    :value (lambda (?psm) 
             (first (role-value ?psm 
                                'has-design-operators)))))
   :own-slots ((tackles-task-type  select-design-operator)))



(def-class COLLECT-FOCUS-OPERATORS (goal-specification-task) ?task
  ((has-input-role :value has-design-focus)
   (has-design-focus :type design-focus)
   ))

(def-class DEFAULT-OPERATOR-COLLECTION (primitive-method) ?psm
  ((has-body 
    :value (lambda (?psm)
             (setofall ?op 
                       (and (design-operator 
                             ?op  
                             applicable-to-parameters ?l)
                            (member (role-value 
                                     ?psm 'has-design-focus)
                                    (eval ?l)))))))
  :own-slots ((tackles-task-type collect-focus-operators)
              (applicability-condition 
               (kappa 
                (?task)
                (and (= :extend 
                        (role-value 
                         ?task 'has-design-context))
                     (parameter 
                      (role-value 
                       ?task 'has-design-focus)))))))


(def-class sort-design-operators (primitive-task) ?task
  ((has-input-role :value has-design-operators
                   :value has-operator-order-relation)
   (has-design-operators :type list)
   (has-operator-order-relation :default-value design-operator-order)
   (has-body 
    :value (lambda (?task)
             (sort (role-value 
                    ?task has-design-operators)
                   (role-value ?task has-operator-order-relation))))))


(def-class SELECT-DESIGN-FOCUS (goal-specification-task) ?task
  ((has-input-role :value has-design-foci)                   
   (has-output-role :value has-design-focus)
   (has-design-foci :type list)
   (has-design-focus :type design-focus)
   (has-goal-expression :value (kappa (?task ?focus)
                               (and ;;(design-focus ?focus)
                                      (has-design-focus ?task ?focus))))))
                                    


(def-class DEFAULT-PARAMETER-SELECTION (primitive-method) 
  ((has-input-role 
    :value has-design-focus-order-relation
    :value has-possible-values-relation)
   (has-design-focus-order-relation 
    :default-value design-focus-order)
   (has-possible-values-relation 
    :default-value possible-value)
   (has-body 
    :value (lambda (?psm)
             (if  (= ?foci (role-value ?psm has-design-foci))
               (select-most-preferred-focus 
                (collect-most-restricted-parameters 
                 ?foci
                 (role-value ?psm 
                             has-possible-values-relation))
                (role-value ?psm 
                            has-design-focus-order-relation))))))
  :own-slots ((tackles-task-type select-design-focus)
              (applicability-condition 
               (kappa (?task)
                      (every (the ?foci 
                               (has-design-foci 
                                ?task ?foci))
                             parameter)))))

;;;;;;;;;;;;;;;;
;;;USE-METHOD STATEMENTS
(tell (use-method consistent-max-state-selection 
                  select-design-state
                  gen-design-psm))

