;;; -*- Mode: LISP; Syntax: Common-lisp; Base: 10; Package: OCML;   -*-

(in-package "OCML")


(in-ontology propose&revise)

(def-class PROPOSE&REVISE (design-method)
  ((has-input-role :value has-design-fixes 
                   :value has-design-procedures)
   (has-design-procedures :type design-procedure)
   (has-design-fixes :type design-fix))
  :lisp-class-name propose&revise)


(def-class FIXABLE-PARAMETRIC-CONSTRAINT (parametric-constraint)
  "A fixable parametric constraint is a parametric constraint which
   can be fixed by some relevant design fixes"
   :Lisp-slots ((relevant-operators :accessor relevant-operators
                                    :initform nil))
    :lisp-class-name  FIXABLE-PARAMETRIC-CONSTRAINT)




(def-class DESIGN-MODIFICATION-OPERATOR (design-operator) 
  "The body of a design modification operator is a 
    binary kappa expressions predicated over a parameter and a value.
   This expression is applied to a parameter and a value and returns a 
   list of assignments and deletions"
  ((body :type design-modification-operator-body))
  :lisp-class-name design-modification-operator )

(def-class DESIGN-MODIFICATION-OPERATOR-BODY (lambda-expression) ?x
  :constraint (and (nth-domain ?x 1 parameter)
                          (nth-domain ?x 3 design-model)
                          (=> (= ?z (call ?x ?p ?v ?d))
                              (and  (has-value ?p ?v ?d)
                                    (== ?z (?assignments ?deletions))
                                    (set ?assignments)
                                    (set ?deletions)
                                    (every (append ?assignments ?deletions)
                                           parameter-assignment)
                                    (every ?deletions
                                           (kappa (?pair)
                                                  (element-of ?pair ?d))))))
   :no-proofs-by (:constraint))
                                                  
                                    


(def-class DESIGN-PROCEDURE (basic-design-extension-operator)
  "Essentially another name for a basic design extension operator"
   :lisp-class-name design-procedure)


(def-class DESIGN-FIX (design-modification-operator)
  "A design fix is a design modification operator which is associated
   with a fixable parametirc constraint"
  ((applicable-to-constraints :type function-expression
                              :documentation "An expression which returns the set
                                              of constraints which can be fixed by 
                                              this fix"))
  :lisp-slots ((effective-constraint-list 
                                         :initform nil
                                         :accessor effective-constraint-list))
   :lisp-class-name design-fix)