;;; Mode: Lisp; Package: ocml

;;; File created in WebOnto

(in-package "OCML")

(in-ontology shipping-patient-service-ontology)




;;as above but with a url




                  #|
(achieve-generic-subtask 
                  ?psm shipping-patient-task
                  'observable1 (role-value ?psm observable1)
                  'observable2 (role-value ?psm observable2)
                  'observable3 (role-value ?psm observable3)
                  'observable4 (role-value ?psm observable4)
                  'patient-location (role-value ?psm patient-location))))
 |#


(defvar *medical-service-for-therapy-mappings*
  '((ocml::arthroplasty . hip-replacement)
    (arthroplasty . hip-replacement)))

(defun medical-service (therapy)
  (cdr (assoc therapy *medical-service-for-therapy-mappings*)))

(defun cleanup-date (date)
  (format nil "~a of ~a, ~d"
          (get-day date)
          (convert-month (get-month date))
          (get-year date)))


;;;all services except shipping patient and diagnose now have an inbuilt
;;;delay specified here
(defvar *shipping-patient-delay* 0)

(defun shipping-patient-delay ()
  (sleep *shipping-patient-delay*))

(def-function run-shipping-patient (?obs1 ?obs2 ?obs3 ?obs4 ?patient-location) 
  "Add an item to magpie collector."
  :lisp-fun
  #'(lambda (obs1 obs2 obs3 obs4 patient-location)
      (run-shipping-patient-service obs1 obs2 obs3 obs4 patient-location)))

(defun run-shipping-patient-service 
       (&optional (obs1 'ocml::severe-joint-pain)
                  (obs2 'ocml::severe-joint-stiffness)
                  (obs3 'ocml::severe-swelling)
                  (obs4 'ocml::joint-creaking)
                  (patient-location 'milton-keynes))
  ;;(setf l (list obs1 obs2 obs3 obs4 patient-location))
  (let ((diagnosis (car (diagnose-patient obs1 obs2 obs3 obs4)))
        therapy hospital-and-date hospital-date hospital medical-service
        hospital-location hospital-currency patient-currency
        cost-in-patient-currency result)
    (format t "The diagnosis for the patient is ~:(~a~)~%" diagnosis)
    (setf therapy (recommend-therapy diagnosis)
          medical-service (medical-service therapy))
    (format t "The recommended therapy for ~:(~a~) is ~:(~a~) which requires ~:(~a~)~%" 
            diagnosis therapy medical-service)
    (setf hospital-and-date  
          (find-soonest-medical-service medical-service)
          hospital (car hospital-and-date)
          hospital-date (cleanup-date (second hospital-and-date))
          hospital-location (get-hospital-location hospital)
          hospital-currency (get-location-currency hospital-location)
          patient-currency (get-location-currency patient-location))
    (format t "The ~(~a~) is located in ~:(~a~) which uses the ~:(~a~).~%" hospital
            hospital-location hospital-currency)
    (format t "Going to ship the patient from ~:(~a~) which uses the ~:(~a~).~%" 
            patient-location patient-currency)
    (setf cost-in-patient-currency
          (ambulance-service-cost patient-location 
                                  hospital-location patient-currency 
                                  hospital-currency))
    (format t "It will cost ~d ~:(~a~) to ship the patient.~%" 
            cost-in-patient-currency patient-currency)
    ;;result is just a string describing what happens to the patient
    ;;
    (setf result (format nil "The patient has ~:(~a~) for which the recommended therapy is ~:(~a~) which requires ~:(~a~). The ~:(~a~) hospital in ~:(~a~) can treat the patient on the ~a. It will cost ~d ~:(~a~) to ship the patient."
                         diagnosis therapy medical-service hospital
                         hospital-location
                         hospital-date cost-in-patient-currency 
                         patient-currency))
    (format t result)
    result))


(defun diagnose-patient (&optional (obs1 'ocml::severe-joint-pain) 
                                   (obs2 'ocml::severe-joint-stiffness)
                                   (obs3 'ocml::severe-swelling)
                                   (obs4 'ocml::joint-creaking))
  (user::achieve-task-through-irs ;;'arthritis-diagnosis-service-ontology
                            ;;no longer need the ontology of the psm, 
                            ;;can use the ontology of the task
                            'fast-arthritis-diagnosis-task
                            'arthritis-diagnosis
                            `(ocml::observable1 ,obs1)
                            `(ocml::observable2 ,obs2)
                            `(ocml::observable3 ,obs3)
                            `(ocml::observable4 ,obs4)))

(defun recommend-therapy (condition)
  (user::achieve-task-through-irs ;;'arthritis-therapy-service-ontology
                            ;;no longer need the ontology of the psm, 
                            ;;can use the ontology of the task
                            'arthritis-therapy-task
                            'arthritis-therapy
                            `(ocml::condition ,condition)))


;;(shipping-patient 'hip-replacement 'milton-keynes)
(defun shipping-patient (medical-service patient-location)
  (let ((hospital (find-soonest-medical-service medical-service))
        hospital-location hospital-currency patient-currency cost-in-patient-currency)
    (setf hospital-location (get-hospital-location hospital)
          hospital-currency (get-location-currency hospital-location)
          patient-currency (get-location-currency patient-location))
    (format t "The ~(~a~) is located in ~:(~a~) which uses the ~:(~a~).~%" hospital
            hospital-location hospital-currency)
    (format t "Going to ship the patient from ~:(~a~) which uses the ~:(~a~).~%" 
            patient-location patient-currency)
    (setf cost-in-patient-currency
          (ambulance-service-cost patient-location hospital-location patient-currency 
                                  hospital-currency))
    (format t "It will cost ~d ~:(~a~) to ship the patient.~%" 
            cost-in-patient-currency patient-currency)))
        

;;'hip-replacement
(defun find-medical-service (service)
  (user::achieve-task-through-irs ;;psm ontology 'medicalservicelocator 
                            ;;task ontology
                            'medical-service-location
                            'medical_service_location
                            (List 'has_medical_service service)))

(defun make-lowercase-string (x)
  (cond ((symbolp x)
         (string-downcase (symbol-name x)))
        ((stringp x)
         (string-downcase x))
        (t (format nil "~(~a~)" x))))

(defun ambulance-service-cost (start destination start-currency destination-currency)
  ;;(setf s start d destination c1 start-currency c2 destination-currency)
  (let* ((amount-in-destination-currency
          (user::achieve-task-through-irs ;;;psm ontology 'GenAmbulanceService 
                                    ;;task ontology
                                    'AmbulanceService
                                    'ambulance_service
                                    (List 'ocml::has_start_location start)
                                    (list 'ocml::has_destination_location destination)
                                    (list 'ocml::has_currency destination-currency)))
        (amount-in-start-currency
         (user::achieve-task-through-irs 
                                   'generic-currency-conversion 
                                   'ocml::generic_currency_conversion
                                   (List 'ocml::has_source_currency 
                                         (make-lowercase-string destination-currency))
                                   (list 'ocml::Has_target_currency 
                                         (make-lowercase-string start-currency))
                                   (list 'ocml::has_amount amount-in-destination-currency))))
    (format t "Have now found an ambulance for ~d ~:(~a~)s or ~d ~:(~a~)s~%" 
            amount-in-destination-currency destination-currency 
            (round amount-in-start-currency) start-currency)
    (round amount-in-start-currency)))



;;'hip-hip-hospital 'hip-replacement
(defun get-hospital-bed-service (hospital-name medical-service)
  (user::achieve-task-through-irs 
   ;;psm ontology 
   ;;'genHospitalBedService
   ;;task ontology
   'hospitalbedservice
   'hospital_bed_service
   (List 'ocml::has_medical_service medical-service)
   (List 'ocml::has_hospital_name hospital-name)))

(defvar *months*
  '("January" "February" "March" "April" "May" "June" "July" "August" "September"
              "October" "November" "December"))

(defun convert-month (num)
  (elt *months* (1- num)))

;;;'hip-replacement
(defun find-soonest-medical-service (service)
  (let* ((hospitals (find-medical-service service))
         result soonest-hospital
         hospitals-and-dates)
    (format t "Found the following hospitals for ~(~a~): ~(~{~a ~}~)~%" service hospitals)
    (setf hospitals-and-dates
          (mapcar #'(lambda (hospital) 
                      (list hospital (get-hospital-bed-service hospital service)))
                  hospitals))
    (setf result
          (sort hospitals-and-dates #'(lambda (x y) (earlier-date-p (second x) (second y))))
          soonest-hospital (car result))
    (format t "~{~{~(~a~) can schedule a patient for the ~d of ~a ~d~%~}~}~%"
            (mapcar #'(lambda (hospital-and-date)
                        (list (car hospital-and-date) (get-day (second hospital-and-date))
                              (convert-month (get-month (second hospital-and-date)))
                              (get-year (second hospital-and-date)))) 
                    result))
    (format t "~(~a~) hospital recommended~%" (car soonest-hospital))
    soonest-hospital))



(defun make-date (day month &optional (year 2004))
  (list (gentemp "DATE")
        'calendar-date
        `(day-of ,day)
        `(month-of ,month)
        `(year-of ,year)))

(defun get-year (date)
  (or (get-date-part 'year-of date)
      ;;why do i need this for some reason when called through the irs
      ;;all of the items appear in the ocml page
      (get-date-part 'user::year-of date)))

(defun get-month (date)
  (or (get-date-part 'month-of date)
      (get-date-part 'user::month-of date)))

(defun get-day (date)
  (or (get-date-part 'day-of date)
      (get-date-part 'user::day-of date)))

(defun get-date-part (part-name date)
  (second (assoc part-name (cddr date))))

(defun earlier-date-p (date1 date2)
  (or (< (get-year date1) (get-year date2))
      (and (= (get-year date1) (get-year date2))
           (or (< (get-month date1) (get-month date2))
               (and (= (get-month date1) (get-month date2))
                    (< (get-day date1) (get-day date2)))))))


(defvar *hospital-locations*
  '((hip-hip-hospital paris)
    (the-hippy-hospital milan)
    (not-the-hippy-hospital amsterdam)
    (another-hippy-hospital barcelona)
    (ocml::hip-hip-hospital paris)
    (ocml::the-hippy-hospital milan)
    (ocml::not-the-hippy-hospital amsterdam)
    (ocml::another-hippy-hospital barcelona)))


(defvar *location-currency*
  '((paris euro)
    (amsterdam euro)
    (barcelona euro)
    (milan euro)
    (milton-keynes pound)))

(defun get-hospital-location (hospital-name)
  (second (assoc hospital-name *hospital-locations*)))


(defun get-location-currency (location)
  (second (assoc location *location-currency*)))

(defun hospital_bed_service_for_hip_replacement (condition)
  (if (eq condition 'hip-replacement) ;;ignore
    (make-date 20 6)
    "FAILURE"))


(defun hip_hip_hospital_bed_service (service hospital)
  hospital ;;ignore
  (shipping-patient-delay)
  (hospital_bed_service_for_hip_replacement service))


(defun  not_the_hippy_hospital_bed_service (service hospital)
  hospital ;;ignore
  (shipping-patient-delay)
  (if (eq service 'hip-replacement) ;;ignore
      "FAILURE"
    (make-date 15 6)))

(defun the_hippy_hospital_bed_service  (service hospital)
  hospital ;;ignore
  (shipping-patient-delay)
  ;;(hospital_bed_service_for_hip_replacement service))
  (if (eq service 'hip-replacement) ;;ignore
    (make-date 7 7)
    "FAILURE"))


(defun another_hippy_hospital_bed_service (service hospital)
    hospital ;;ignore
    (shipping-patient-delay)
    ;;(hospital_bed_service_for_hip_replacement service)
  (if (eq service 'hip-replacement) ;;ignore
    (make-date 13 8)
    "FAILURE"))