package org.infrawebs.axiom.editor;

import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.actions.GEFActionConstants;
import org.eclipse.jface.action.IAction;
import org.eclipse.jface.action.IMenuManager;
import org.eclipse.ui.IWorkbenchActionConstants;
import org.eclipse.ui.actions.ActionFactory;
import org.infrawebs.axiom.editor.actions.AxiomAction;
import org.infrawebs.axiom.editor.actions.ConnectToExistingAction;
import org.infrawebs.axiom.editor.actions.RenameAction;
import org.infrawebs.axiom.editor.actions.connections.*;
import org.infrawebs.axiom.editor.actions.contents.*;
import org.infrawebs.axiom.editor.actions.memberofs.*;
import org.infrawebs.axiom.editor.actions.ontology.*;
import org.infrawebs.axiom.editor.actions.operators.*;
import org.infrawebs.axiom.editor.actions.refinables.*;
import org.infrawebs.axiom.editor.actions.visual.*;

public class ContextMenuProviderImpl extends ContextMenuProvider
{
	private ActionRegistry actionRegistry;
	private IMenuManager manager; 
	
	public ContextMenuProviderImpl(ActionRegistry ar, GraphicalViewer gv)
	{
		super(gv);
		this.actionRegistry = ar;
	}
	
	private void addAction(String actionId, String groupName, boolean showDisabled) {
		IAction action = this.actionRegistry.getAction(actionId);
		if (action.isEnabled() || showDisabled) {
			this.manager.appendToGroup(groupName, action);
		}
	}
	
	private void addAxiomAction(Class actionClass, String groupName) {
		AxiomAction action = (AxiomAction)this.actionRegistry.getAction(
				actionClass.getName());
		action.update();
		if (action.calculateVisible()) {
			this.manager.appendToGroup(groupName, action);
		}
	}

	@Override
	public void buildContextMenu(IMenuManager menuManager)
	{
		this.manager = menuManager;
		
		GEFActionConstants.addStandardActionGroups(menuManager);

		addAction(ActionFactory.UNDO.getId(), GEFActionConstants.GROUP_UNDO, true);
		addAction(ActionFactory.REDO.getId(), GEFActionConstants.GROUP_UNDO, true);

		addAxiomAction(CreateVariableAction.class, GEFActionConstants.GROUP_EDIT);
		addAxiomAction(CreateInstanceAction.class, GEFActionConstants.GROUP_EDIT);
		addAxiomAction(CreateRelationAction.class, GEFActionConstants.GROUP_EDIT);
		
		addAxiomAction(CreateOperatorOrAction.class, GEFActionConstants.GROUP_EDIT);
		addAxiomAction(CreateOperatorAndAction.class, GEFActionConstants.GROUP_EDIT);
		addAxiomAction(CreateOperatorNotAction.class, GEFActionConstants.GROUP_EDIT);
		
		addAxiomAction(InsertOperatorAndAction.class, GEFActionConstants.GROUP_EDIT);		
		addAxiomAction(InsertOperatorOrAction.class, GEFActionConstants.GROUP_EDIT);
		addAxiomAction(InsertOperatorNotAction.class, GEFActionConstants.GROUP_EDIT);
		
		addAxiomAction(RefineByDefaultTypeVariableAction.class, GEFActionConstants.GROUP_EDIT);
		addAxiomAction(RefineByDefaultTypeValueInstanceAction.class, GEFActionConstants.GROUP_EDIT);
		addAxiomAction(RefineByCustomTypeVariableAction.class, GEFActionConstants.GROUP_EDIT);
		addAxiomAction(RefineByCustomTypeValueInstanceAction.class, GEFActionConstants.GROUP_EDIT);		
		addAxiomAction(RefineByCustomTypeComplexInstanceAction.class, GEFActionConstants.GROUP_EDIT);
		addAxiomAction(RefineAndInvolveInRelationAction.class, GEFActionConstants.GROUP_EDIT);
		addAxiomAction(AdditionalAttributeValueAction.class, GEFActionConstants.GROUP_EDIT);

		addAxiomAction(AddOperandDefaultTypeVariableAction.class, GEFActionConstants.GROUP_EDIT);
		addAxiomAction(AddOperandDefaultTypeValueInstanceAction.class, GEFActionConstants.GROUP_EDIT);
		addAxiomAction(AddOperandCustomTypeVariableAction.class, GEFActionConstants.GROUP_EDIT);
		addAxiomAction(AddOperandCustomTypeValueInstanceAction.class, GEFActionConstants.GROUP_EDIT);
		addAxiomAction(AddOperandCustomTypeComplexInstanceAction.class, GEFActionConstants.GROUP_EDIT);
		addAxiomAction(AddOperandRelationAction.class, GEFActionConstants.GROUP_EDIT);
		
		addAxiomAction(InvolveInNewRelationAction.class, GEFActionConstants.GROUP_EDIT);
		
		addAxiomAction(RenameAction.class, GEFActionConstants.GROUP_VIEW);
		addAxiomAction(ConnectToExistingAction.class, GEFActionConstants.GROUP_VIEW);
		addAction(ActionFactory.DELETE.getId(), GEFActionConstants.GROUP_VIEW, false);
		
		addAxiomAction(LocateInOntoViewAction.class, GEFActionConstants.GROUP_FIND);
		addAxiomAction(LocateTypeInOntoViewAction.class, GEFActionConstants.GROUP_FIND);

		addAxiomAction(HideUnusedAttributes.class, IWorkbenchActionConstants.GROUP_ADD);
		addAxiomAction(BringToFrontAction.class, IWorkbenchActionConstants.GROUP_ADD);
		addAxiomAction(SendToBackAction.class, IWorkbenchActionConstants.GROUP_ADD);
	}

}
