package org.infrawebs.axiom.editor.actions;

import org.eclipse.gef.editparts.AbstractEditPart;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IWorkbenchPart;
import org.infrawebs.axiom.editor.AxiomConnectionTool;
import org.infrawebs.axiom.editor.editparts.EditPartWithConnections;
import org.infrawebs.axiom.model.AbstractModel;
import org.infrawebs.axiom.model.ModelWithConnections;
import org.infrawebs.axiom.operations.SemanticConsistence;
import org.infrawebs.axiom.settings.ImageSettings;

public class ConnectToExistingAction extends AxiomAction
{
	public ConnectToExistingAction(IWorkbenchPart part) {
		super(part);
	}

	@Override
	public boolean calculateVisible() {
		AbstractModel model = getSelectedModel();
		return (model instanceof ModelWithConnections);
	}
	
	@Override
	public boolean calculateEnabled() {
		AbstractEditPart editPart = getSelectedEditPart();
		// Require that the editpart is an EditPartWithConnections
		// and NOT a TreeEditPartWithConnections.
		return (editPart instanceof EditPartWithConnections) &&
				SemanticConsistence.allowMoreChildren((ModelWithConnections)getSelectedModel());
	}
	
	@Override
	protected String getCaption() {
		return "Connect to...";
	}

	@Override
	protected Image getImage() {
		return ImageSettings.getImage(ImageSettings.CONNECTION);
	}

	@Override
	public void run() {
		if (!calculateEnabled()) {
			return;
		}
		EditPartWithConnections sourcePart = (EditPartWithConnections)this.getSelectedEditPart();
		new AxiomConnectionTool(sourcePart);
	}

}
