package org.infrawebs.axiom.editor.actions.memberofs;

import java.util.HashSet;
import java.util.List;

import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IWorkbenchPart;
import org.infrawebs.axiom.editor.actions.ontology.SelectFromOntologyAction;
import org.infrawebs.axiom.model.AbstractModel;
import org.infrawebs.axiom.model.MemberOfModel;
import org.infrawebs.axiom.model.ModelWithConnections;
import org.infrawebs.axiom.model.ParameterModel;
import org.infrawebs.axiom.model.RelationModel;
import org.infrawebs.axiom.operations.ModelOperations;
import org.infrawebs.axiom.operations.SemanticConsistence;
import org.infrawebs.ontology.OntoFilter;
import org.infrawebs.ontology.shells.ParameterShell;
import org.infrawebs.ontology.ui.OntoImages;
import org.omwg.ontology.OntologyElement;
import org.omwg.ontology.Type;

public class InvolveInNewRelationAction extends SelectFromOntologyAction
{
	public InvolveInNewRelationAction(IWorkbenchPart part) {
		super(part);
	}

	@Override
	public boolean calculateVisible() {
		AbstractModel model = getSelectedModel();
		return (model instanceof MemberOfModel);
	}

	@Override
	public boolean calculateEnabled() {
		AbstractModel model = getSelectedModel();
		return (model instanceof MemberOfModel) &&
				SemanticConsistence.allowMoreParents((ModelWithConnections)model);
	}

	@Override
	protected String getCaption() {
		return "Involve in relation";
	}

	@Override
	protected Image getImage() {
		return OntoImages.getImage(OntoImages.RELATION);
	}

	@Override
	protected String getSelectionDialogTitle() {
		return "Please, select a compatible relation parameter.";
	}

	@Override
	protected OntoFilter createOntoFilter() {
		OntoFilter ontoFilter = new OntoFilter(false);

		MemberOfModel memberOfModel = (MemberOfModel)getSelectedModel();
		ontoFilter.baseTypes = new HashSet<Type>();
		ontoFilter.baseTypes.add(memberOfModel.getConcept());

		ontoFilter.showRelations = true;
		ontoFilter.selectParameters = true;
		return ontoFilter;
	}

	public static ParameterModel findParameterModel(RelationModel relationModel, ParameterShell paramInfo) {
		// Find among the parameter models the one that corresponds
		// to the parameter which the user selected in the dialog.
		List paramModels = relationModel.getRefinableList().getChildren();
		for (int i = 0; i < paramModels.size(); i++) {
			ParameterModel paramModel = (ParameterModel)paramModels.get(i);
			if (paramModel.getRefinable().equals(paramInfo)) {
				return paramModel;
			}
		}
		return null;
	}

	@Override
	protected void onOkPressed(OntologyElement ontologyElement) {
		// Creates a model for the relation which the selected parameter belongs to.
		ParameterShell paramInfo = (ParameterShell)ontologyElement;
		RelationModel relationModel = getContentsModel().getFactory().createRelation(paramInfo);
		if (relationModel == null) {
			return;
		}
		
		MemberOfModel memberOfModel = (MemberOfModel)getSelectedModel();
		ParameterModel paramModel = findParameterModel(relationModel, paramInfo);
		// Connect that parameter model to the active MemberOfModel.
		ModelOperations.createAndConnect(relationModel, paramModel, memberOfModel);
	}
}
