package org.infrawebs.axiom.editor.actions.ontology;

import org.eclipse.jface.window.Window;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPart;
import org.infrawebs.axiom.editor.actions.AxiomAction;
import org.infrawebs.axiom.plugin.AxiomEditorPlugin;
import org.infrawebs.ontology.OntoFilter;
import org.infrawebs.ontology.ui.OntoSelectionDialog;
import org.omwg.ontology.OntologyElement;

public abstract class SelectFromOntologyAction extends AxiomAction
{
	public SelectFromOntologyAction(IWorkbenchPart part) {
		super(part);
	}

	protected abstract String getSelectionDialogTitle();

	// Successors must implement this method to show only the
	// parts of the ontology relevant to the type of action.
	protected abstract OntoFilter createOntoFilter();

	// Successors must override this to execute custom commands.
	protected abstract void onOkPressed(OntologyElement ontologyElement);
	
	@Override
	public void run() {
		if (!calculateEnabled()) {
			return;
		}
		Shell shell = AxiomEditorPlugin.getDefault().getWorkbench().
				getActiveWorkbenchWindow().getShell();
		OntoSelectionDialog dialog = new OntoSelectionDialog(
				shell, AxiomEditorPlugin.ontoStore, this.createOntoFilter());
		dialog.setTitle(this.getSelectionDialogTitle());
		dialog.open();
		dialog.close();

		if (dialog.getReturnCode() == Window.OK) {
			onOkPressed(dialog.getOntologySelection());
		}
	}

}
