package org.infrawebs.axiom.editor.actions.refinables;

import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IWorkbenchPart;
import org.infrawebs.axiom.editor.actions.AxiomAction;
import org.infrawebs.axiom.model.AbstractModel;
import org.infrawebs.axiom.model.AttributeListModel;
import org.infrawebs.axiom.model.AttributeModel;
import org.infrawebs.axiom.model.ModelWithConnections;
import org.infrawebs.axiom.model.RefinableListModel;
import org.infrawebs.axiom.model.RefinableModel;
import org.infrawebs.axiom.model.commands.ContainerAddChildCommand;
import org.infrawebs.axiom.operations.ModelOperations;
import org.infrawebs.ontology.shells.AttributeShell;
import org.infrawebs.ontology.ui.OntoImages;

public class AdditionalAttributeValueAction extends AxiomAction
{
	public AdditionalAttributeValueAction(IWorkbenchPart part) {
		super(part);
	}

	@Override
	protected String getCaption() {
		return "Additional Attribute Value";
	}

	@Override
	protected Image getImage() {
		return OntoImages.getImage(OntoImages.ATTRIBUTE_NEW);
	}
	
	@Override
	protected boolean calculateEnabled() {
		AbstractModel selectedModel = getSelectedModel();
		if (!(selectedModel instanceof AttributeModel)) {
			return false;
		}
		
		// Only enabled when this attribute's count < MaxCardinality of this attribute.
		AttributeModel model = (AttributeModel)selectedModel;
		AttributeShell shell = (AttributeShell)model.getRefinable();
		AttributeListModel list = (AttributeListModel)model.getOwner();
		int count = 0;
		for (ModelWithConnections child: list.getChildren()) {
			AttributeModel childModel = (AttributeModel)child;
			if (childModel.getRefinable().equals(shell)) {
				count++;
			}
		}
		return (count < shell.getAttribute().getMaxCardinality());
	}
	
	@Override
	public boolean calculateVisible() {
		AbstractModel model = getSelectedModel();
		return (model instanceof AttributeModel) &&
				(((RefinableModel)model).getRefiningModel() != null);
	}
	
	@Override
	public void run() {
		if (!calculateEnabled()) {
			return;
		}
		
		AttributeModel source = (AttributeModel)getSelectedModel();
		RefinableListModel list = source.getOwner();
		AttributeShell shell = (AttributeShell)source.getRefinable();
		AttributeModel clone = new AttributeModel(getContentsModel(), shell);

		ModelOperations.executeCommand(new ContainerAddChildCommand(list, clone, source));
	}

}
