package org.infrawebs.axiom.editor.actions.refinables;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.ui.IWorkbenchPart;
import org.infrawebs.axiom.editor.actions.ontology.SelectFromOntologyAction;
import org.infrawebs.axiom.model.AbstractModel;
import org.infrawebs.axiom.model.MemberOfModel;
import org.infrawebs.axiom.model.RefinableModel;
import org.infrawebs.axiom.operations.ModelOperations;
import org.infrawebs.ontology.OntoFilter;
import org.omwg.ontology.OntologyElement;

public abstract class RefineByCustomTypeAction extends SelectFromOntologyAction
{
	public RefineByCustomTypeAction(IWorkbenchPart part) {
		super(part);
	}

	@Override
	public boolean calculateVisible() {
		AbstractModel model = getSelectedModel();
		return (model instanceof RefinableModel) &&
				(((RefinableModel)model).getRefiningModel() == null);
	}
	
	@Override
	protected OntoFilter createOntoFilter() {
		OntoFilter ontoFilter = new OntoFilter(false);
		RefinableModel refinable = (RefinableModel)getSelectedModel();
		ontoFilter.baseTypes = refinable.getRefinable().listTypes();
		return ontoFilter;
	}
	
	@Override
	protected void onOkPressed(OntologyElement ontologyElement) {
		MemberOfModel memberOfModel = (MemberOfModel)getContentsModel().getFactory().createAxiomElement(ontologyElement);
		if (memberOfModel == null) {
			return;
		}
		
		// Place the new figure to the right of the refinable's variable.
		RefinableModel sourceModel = (RefinableModel)getSelectedModel();
		// The paranet model is either a VariableModel or a RelationModel.
		AbstractModel parentModel = sourceModel.getOwner().getOwner();
		Rectangle rect = (Rectangle)parentModel.getConstraint();
		memberOfModel.setConstraint(new Rectangle(rect.x + 200, rect.y, -1, -1));

		ModelOperations.createAndConnect(
				memberOfModel, sourceModel, memberOfModel);
	}

}
