package org.infrawebs.axiom.editor.actions.refinables;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.ui.IWorkbenchPart;
import org.infrawebs.axiom.editor.actions.AxiomAction;
import org.infrawebs.axiom.model.AbstractModel;
import org.infrawebs.axiom.model.AxiomContentsModel;
import org.infrawebs.axiom.model.MemberOfModel;
import org.infrawebs.axiom.model.ModelWithConnections;
import org.infrawebs.axiom.model.RefinableModel;
import org.infrawebs.axiom.operations.ModelOperations;
import org.infrawebs.axiom.plugin.AxiomEditorPlugin;
import org.omwg.ontology.Concept;
import org.omwg.ontology.Type;

public abstract class RefineByDefaultTypeAction extends AxiomAction
{
	public RefineByDefaultTypeAction(IWorkbenchPart part) {
		super(part);
	}

	@Override
	public boolean calculateVisible() {
		AbstractModel model = getSelectedModel();
		return (model instanceof RefinableModel) &&
				(((RefinableModel)model).getRefiningModel() == null);
	}
	
	public static MemberOfModel createDefaultRefiningModel(AxiomContentsModel contents, RefinableModel refinableModel) {
		Type defaultType = refinableModel.getRefinable().getFirstType();
		Concept concept = AxiomEditorPlugin.ontoStore.liftTypeToConcept(defaultType); 
		MemberOfModel memberOfModel = (MemberOfModel)contents.getFactory().createAxiomElement(concept);
		if (memberOfModel == null) {
			return null;
		}

		// Place the new figure to the right of the refinable's variable.
		// The paranet model is either a VariableModel or a RelationModel.
		ModelWithConnections parentModel = refinableModel.getOwner().getOwner();
		Rectangle rect = (Rectangle)parentModel.getConstraint();
		memberOfModel.setConstraint(new Rectangle(rect.x + 200, rect.y, -1, -1));
		
		return memberOfModel;
	}
	
	@Override
	public void run() {
		if (!calculateEnabled()) {
			return;
		}
		
		RefinableModel refinableModel = (RefinableModel)getSelectedModel();
		MemberOfModel memberOfModel = createDefaultRefiningModel(getContentsModel(), refinableModel);
		if (memberOfModel == null) {
			return;
		}
		ModelOperations.createAndConnect(
				memberOfModel, refinableModel, memberOfModel);
	}

}
