package org.infrawebs.axiom.editor.actions.visual;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CompoundCommand;
import org.eclipse.swt.graphics.Image;
import org.eclipse.ui.IWorkbenchPart;
import org.infrawebs.axiom.editor.actions.AxiomAction;
import org.infrawebs.axiom.model.AttributeListModel;
import org.infrawebs.axiom.model.AttributeModel;
import org.infrawebs.axiom.model.ModelWithConnections;
import org.infrawebs.axiom.model.VariableModel;
import org.infrawebs.axiom.model.commands.ContainerDeleteChildCommand;
import org.infrawebs.axiom.operations.ModelOperations;
import org.infrawebs.ontology.ui.OntoImages;

public class HideUnusedAttributes extends AxiomAction
{
	public HideUnusedAttributes(IWorkbenchPart part) {
		super(part);
	}

	@Override
	public boolean calculateVisible() {
		return (getSelectedModel() instanceof VariableModel);
	}
	
	@Override
	public void run() {
		if (!calculateEnabled()) {
			return;
		}
		VariableModel selectedModel = (VariableModel)getSelectedModel();
		AttributeListModel list = selectedModel.getRefinableList();
		
		CompoundCommand compoundCommand = new CompoundCommand();
		for (ModelWithConnections child: list.getChildren()) {
			AttributeModel childModel = (AttributeModel)child;
			if (childModel.getVarNameModel() == null) {
				Command cmd = new ContainerDeleteChildCommand(list, childModel);
				compoundCommand.add(cmd);
			}
		}
		ModelOperations.executeCommand(compoundCommand);

	}

	@Override
	protected String getCaption() {
		return "Hide Unused Attributes";
	}

	@Override
	protected Image getImage() {
		return OntoImages.getImage(OntoImages.ATTRIBUTE_NEW_DISABLED);
	}

}
