package org.infrawebs.axiom.editor.editparts;

import java.beans.PropertyChangeEvent;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.jface.viewers.TextCellEditor;
import org.infrawebs.axiom.editor.editpolicies.DeleteMemberOfEditPolicy;
import org.infrawebs.axiom.editor.figures.MemberOfFigure;
import org.infrawebs.axiom.editor.figures.RatioConnectionAnchor;
import org.infrawebs.axiom.model.Compactable;
import org.infrawebs.axiom.model.MemberOfModel;
import org.infrawebs.axiom.model.VarNameHolderModel;
import org.infrawebs.axiom.model.VarNameModel;

public abstract class MemberOfEditPart extends DirectEditEditPart
{
	@Override
	protected void createEditPolicies() {
		super.createEditPolicies();		
		// Takes care of disableing VarNameModels when all their holders are deleted.
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new DeleteMemberOfEditPolicy());
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		MemberOfModel model = (MemberOfModel)getModel();
		if (evt.getPropertyName().equals(Compactable.ID_PROPERTY_COMPACT)) {
			// If the "compactness" of the model has just changed, change the figure.
			MemberOfFigure localFigure = (MemberOfFigure)getFigure();
			if (model.isCompact() != localFigure.isCompact()) {
				localFigure.setCompact(model.isCompact(), model.getConstraint());
			}
		}
		else if (evt.getPropertyName().equals(MemberOfModel.ID_PROPERTY_TYPE)) {
			MemberOfFigure localFigure = (MemberOfFigure)getFigure();
			String conceptName = (String)evt.getNewValue();
			localFigure.setConceptName(conceptName);
		}
		else if (evt.getPropertyName().equals(VarNameModel.ID_PROPERTY_VARNAME)) {
			MemberOfFigure localFigure = (MemberOfFigure)getFigure();
			String variableName = (String)evt.getNewValue();
			localFigure.getVarNameFigure().setText(variableName);
		}
		else if (evt.getPropertyName().equals(VarNameHolderModel.ID_PROPERTY_VARNAME_MODEL)) {
			MemberOfFigure localFigure = (MemberOfFigure)getFigure();
			VarNameModel varNameModel = (VarNameModel)evt.getNewValue();
			localFigure.getVarNameFigure().setText(varNameModel.getName());
		}
		else if (evt.getPropertyName().equals(VarNameModel.ID_PROPERTY_EXISTS_FLAG)) {
			MemberOfFigure localFigure = (MemberOfFigure)getFigure();
			boolean existsFlag = ((Boolean)evt.getNewValue()).booleanValue();
			localFigure.getVarNameFigure().setExistsFlag(existsFlag);
		}
		else {
			super.propertyChange(evt);
		}
	}

	@Override
	protected void refreshVisuals() {
		super.refreshVisuals();
 		MemberOfModel model = (MemberOfModel)getModel();
		MemberOfFigure localFigure = (MemberOfFigure)getFigure();
		localFigure.setEnabled(model.isFixedInExpression());
	}

	@Override
	protected void refreshTargetConnections() {
		super.refreshTargetConnections();
		refreshVisuals();
	}

	// Creates a text editor over the variable type label.
	@Override
	protected AxiomDirectEditManager createDirectEditManager() {
		MemberOfModel model = (MemberOfModel)getModel();
		if (!model.isCompact()) {
			IFigure directEditFigure = ((MemberOfFigure)getFigure()).getDirectEditFigure();
			if (directEditFigure != null) {
				AxiomCellEditorLocator locator = new AxiomCellEditorLocator(directEditFigure);
				return new AxiomDirectEditManager(this, TextCellEditor.class, locator);
			}
		}
		return null;
	}

	@Override
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
		return new RatioConnectionAnchor(getFigure(), 0.3, 0.0);		
	}

	@Override
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		return new RatioConnectionAnchor(getFigure(), 0.3, 0.0);
	}

	/**
	 * Returns the panel containing the child figures (attributes/parameters).
	 */
	@Override
	public IFigure getContentPane() {
		MemberOfFigure localFigure = (MemberOfFigure)getFigure();
		return localFigure.getBodyPanel();
	}

}
