package org.infrawebs.axiom.editor.editparts;

import java.beans.PropertyChangeEvent;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.EllipseAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.infrawebs.axiom.editor.editpolicies.DeleteWithConnectionsEditPolicy;
import org.infrawebs.axiom.editor.figures.OperatorFigure;
import org.infrawebs.axiom.model.OperatorModel;
import org.infrawebs.axiom.settings.ColorSettings;


public class OperatorEditPart extends EditPartWithConnections
{
	@Override
	protected IFigure createFigure() {
		return new OperatorFigure();
	}

	@Override
	protected void createEditPolicies() {
		super.createEditPolicies();		
		// Takes care of deletion of nodes and their connections.
		installEditPolicy(EditPolicy.COMPONENT_ROLE, new DeleteWithConnectionsEditPolicy());
	}

	@Override
	protected void refreshVisuals() {
		super.refreshVisuals();
		OperatorModel model = (OperatorModel)getModel();
		OperatorFigure localFigure = (OperatorFigure)getFigure();
		localFigure.setOperatorTypeAsString(model.getOperatorTypeAsString());
		localFigure.setBackgroundColor(
				model.hasEnoughOperands() && model.getParent() != null
				? ColorSettings.OPERATOR_BACK
				: ColorSettings.OPERATOR_BACK_DISABLED);
	}

	@Override
	protected void refreshSourceConnections() {
		super.refreshSourceConnections();
		refreshVisuals();
	}

	@Override
	protected void refreshTargetConnections() {
		super.refreshTargetConnections();
		refreshVisuals();
	}

	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals(OperatorModel.ID_PROPERTY_OPERATOR_TYPE)) {
			refreshVisuals();
		}
		else {
			super.propertyChange(evt);
		}
	}

	@Override
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		return new EllipseAnchor(getFigure());
	}

	@Override
	public ConnectionAnchor getTargetConnectionAnchor(ConnectionEditPart connection) {
		return new EllipseAnchor(getFigure());
	}

	@Override
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		return new EllipseAnchor(getFigure());
	}

	@Override
	public ConnectionAnchor getTargetConnectionAnchor(Request request) {
		return new EllipseAnchor(getFigure());
	}

}
