package org.infrawebs.axiom.editor.editparts;

import java.beans.PropertyChangeEvent;

import org.eclipse.draw2d.ConnectionAnchor;
import org.eclipse.draw2d.IFigure;
import org.eclipse.gef.ConnectionEditPart;
import org.eclipse.gef.Request;
import org.eclipse.swt.graphics.Font;
import org.infrawebs.axiom.editor.figures.RatioConnectionAnchor;
import org.infrawebs.axiom.editor.figures.RefinableFigure;
import org.infrawebs.axiom.model.AttributeModel;
import org.infrawebs.axiom.model.RefinableModel;
import org.infrawebs.axiom.model.VarNameHolderModel;
import org.infrawebs.axiom.model.VarNameModel;
import org.infrawebs.axiom.settings.FontSettings;
import org.infrawebs.common.Util;
import org.infrawebs.ontology.shells.AttributeShell;
import org.infrawebs.ontology.ui.OntoImages;
import org.omwg.ontology.Type;

public abstract class RefinableEditPart extends EditPartWithConnections
{
	@Override
	public IFigure createFigure() {
		RefinableModel refinable = (RefinableModel)getModel();
		String shortName = Util.getLocalName(refinable.getRefinable());
		return new RefinableFigure(shortName);
	}

	@Override
	protected void refreshVisuals() {
		super.refreshVisuals();
		
		String suffix;
		int iconIndex;
		Font font;

		RefinableModel refinable = (RefinableModel)getModel();
		if (refinable.getVarNameModel() != null) {
			
			suffix = "";
			
			// append the Maximum Cardinality if greater than 1
			if ((refinable instanceof AttributeModel) &&
				(((AttributeShell)refinable.getRefinable()).getAttribute().getMaxCardinality() > 1)) { 
					suffix +=  "(" + ((AttributeShell)refinable.getRefinable()).getMaxCardinalityAsString() + ") ";
			}
			
			// Append the name of the associated variable.
			suffix  += "= ?" + refinable.getVarNameModel().getName();

			font = FontSettings.REFINABLE_CONNECTED;
			iconIndex = getImageIndexEnabled();
		}
		else {
			// Append the short name of the fisrt type just for a clue.
			Type firstType = refinable.getRefinable().getFirstType();
			suffix = "[" + Util.getLocalName(firstType);
			
/*			Da pokazva na nesvyrzanite atributi kolko im e MaxCardinality
 			if (refinable instanceof AttributeModel) {
				suffix +=  " " + ((AttributeShell)refinable.getRefinable()).getMaxCardinalityAsString();				
			}
*/
			suffix += "]";				
			font = FontSettings.REFINABLE_DISCONNECTED;
			iconIndex = getImageIndexDisabled();
		}
		
		RefinableFigure localFigure = (RefinableFigure)getFigure();
		localFigure.setSuffix(suffix);
		localFigure.setFont(font);
		localFigure.setIcon(OntoImages.getImage(iconIndex));
	}

	@Override
	protected void refreshSourceConnections() {
		super.refreshSourceConnections();
		refreshVisuals();
	}
	
	@Override
	public void propertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals(VarNameModel.ID_PROPERTY_VARNAME)) {
			refreshVisuals();
		}
		else if (evt.getPropertyName().equals(VarNameHolderModel.ID_PROPERTY_VARNAME_MODEL)) {
			refreshVisuals();
		}
		else {
			super.propertyChange(evt);
		}
	}
	
	@Override
	public ConnectionAnchor getSourceConnectionAnchor(ConnectionEditPart connection) {
		return new RatioConnectionAnchor(getFigure(), 1.0, 0.5);		
		
//		Point ptRight = getFigure().getBounds().getRight();
//		return new XYAnchor(ptRight);

//		Point ptTargetCenter = ((EditPartWithConnections)connection.getTarget()).getFigure().getBounds().getCenter();
	}

	@Override
	public ConnectionAnchor getSourceConnectionAnchor(Request request) {
		return new RatioConnectionAnchor(getFigure(), 1.0, 0.5);

//		Point ptRight = getFigure().getBounds().getRight();
//		return new XYAnchor(ptRight);
		
//		Point pt = new Point(((DropRequest)request).getLocation());
//		Point ptRight = getFigure().getBounds().getRight();
//		Point ptLeft = getFigure().getBounds().getLeft();
//		if (pt.getDistance2(ptLeft) < pt.getDistance2(ptRight))
//			return new XYAnchor(ptLeft);
//		else
//			return new XYAnchor(ptRight);
	}

	protected abstract int getImageIndexEnabled();
	protected abstract int getImageIndexDisabled();

}
