package org.infrawebs.axiom.editor.figures;

import org.eclipse.draw2d.Clickable;
import org.eclipse.draw2d.ColorConstants;
import org.eclipse.draw2d.Cursors;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.MarginBorder;
import org.eclipse.jface.util.Assert;
import org.eclipse.swt.graphics.Color;

public class HyperLinkLabel extends Clickable
{
	private InnerLabel innerLabel;
	Color hoverColor, normalColor;
	
	private class InnerLabel extends Label {
		public InnerLabel(String text) {
			super(text);
		}

	    @Override
	    protected void paintFigure(Graphics graphics) {
	      if (getModel().isMouseOver() && getText().length() != 0) {
	        setForegroundColor(HyperLinkLabel.this.hoverColor);
	        super.paintFigure(graphics);
	        graphics.drawLine(getTextBounds().getBottomLeft(),
	            getTextBounds().getBottomRight());
	      } else {
	        setForegroundColor(HyperLinkLabel.this.normalColor);
	        super.paintFigure(graphics);
	      }
	    }
	}

	public HyperLinkLabel(String label) {
		Assert.isNotNull(label);
		this.innerLabel = new InnerLabel(label);
		this.innerLabel.setBorder(new MarginBorder(2));
		setCursor(Cursors.HAND);
		setContents(this.innerLabel);

		setRolloverEnabled(true);

		this.normalColor = ColorConstants.blue;
		this.hoverColor = ColorConstants.red;
	}

	public Color getNormalColor() {
		return this.normalColor;
	}

	public void setNormalColor(Color color) {
		this.normalColor = color;
		repaint();
	}

	public Color getHoverColor() {
		return this.hoverColor;
	}

	public void setHoverColor(Color color) {
		this.hoverColor = color;
	}

}
