package org.infrawebs.axiom.editor.figures;

import org.eclipse.draw2d.AbstractBorder;
import org.eclipse.draw2d.Graphics;
import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.geometry.*;
import org.eclipse.draw2d.*;
import org.eclipse.swt.graphics.Color;


public class ShadowBorder extends AbstractBorder
{
	private int width = 3;
	private Color [] colors; 
	
	public void setWidth(int width)
	{
		this.width = width;
		this.colors = null;
	}

	private Color [] getColors()
	{
		if (this.colors == null) {
			createColors();
		}
		return this.colors;
	}

	private void createColors()
	{
		Color ref1 = ColorConstants.lightGray;
		Color ref2 = ColorConstants.white;
		this.colors = new Color[this.width];
		for(int i = 0; i < this.width; ++i)
			this.colors[this.width - 1 - i] = new Color(null, ref2.getRed() + (ref1.getRed() - ref2.getRed()) * i / (this.width - 1),
				ref2.getGreen() + (ref1.getGreen() - ref2.getGreen()) * i / (this.width - 1),
				ref2.getBlue() + (ref1.getBlue() - ref2.getBlue()) * i / (this.width - 1));
	}

	public Insets getInsets(IFigure arg0)
	{
		return new Insets(0, 0, this.width, this.width);
	}

	public void paint(IFigure arg0, Graphics arg1, Insets arg2)
	{
		Rectangle bounds = arg0.getBounds();
		Rectangle rectExt = bounds.getCropped(arg2);
		Color backg = arg0.getParent().getBackgroundColor();
		arg1.setBackgroundColor(backg);
		arg1.fillRectangle(rectExt.x + rectExt.width-this.width, rectExt.y, this.width, this.width);
		arg1.fillRectangle(rectExt.x, rectExt.y+rectExt.height - this.width, this.width, this.width);
		
		int lVert = rectExt.height - this.width - 1;
		int lHor =  rectExt.width - this.width - 1;
		Point coinInfDroit = new Point(rectExt.x+rectExt.width-1, rectExt.y+rectExt.height-1);
		Point coinSupDroit = coinInfDroit.getCopy().translate(0,-lVert);
		Point coinInfGauche = coinInfDroit.getCopy().translate(-lHor,0);
		for(int i = 0; i < this.width;++i)
		{
			arg1.setForegroundColor(getColors()[i]);
			arg1.drawLine(coinInfDroit, coinSupDroit);
			arg1.drawLine(coinInfDroit, coinInfGauche);
			coinInfDroit.translate(-1,-1);
			coinSupDroit.translate(-1,-1);
			coinInfGauche.translate(-1,-1);
		}
	}
}
