package org.infrawebs.axiom.editor.inputs;

import org.eclipse.core.runtime.IPath;
import org.eclipse.jface.resource.ImageDescriptor;
import org.eclipse.ui.IPathEditorInput;
import org.eclipse.ui.IPersistableElement;
import org.eclipse.ui.PlatformUI;

/**
 * EditorInput that stores a path.
 */
public class PathEditorInput implements IPathEditorInput
{
	private IPath path = null;

	/**
	 * Creates an editor input based of the given file resource.
	 */
	public PathEditorInput(IPath path) {
		if (path == null) {
			throw new IllegalArgumentException();
		}
		this.path = path;
	}

	@Override
	public int hashCode() {
		return this.path.hashCode();
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (!(obj instanceof PathEditorInput))
			return false;
		PathEditorInput other = (PathEditorInput) obj;
		return this.path.equals(other.path);
	}

	public boolean exists() {
		return this.path.toFile().exists();
	}

	public ImageDescriptor getImageDescriptor() {
		return PlatformUI.getWorkbench().getEditorRegistry().
				getImageDescriptor(this.path.toString());
	}

	public String getName() {
		return this.path.removeFileExtension().toString();
	}

	public String getToolTipText() {
		return getName();
	}

	public IPath getPath() {
		return this.path;
	}

	public Object getAdapter(Class adapter) {
		return null;
	}

	public IPersistableElement getPersistable() {
		// no persistence
		return null;
	}

}
