package org.infrawebs.axiom.logexpr;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Set;

import org.eclipse.jface.window.Window;
import org.infrawebs.common.Util;
import org.omwg.logicalexpression.LogicalExpression;
import org.omwg.ontology.Axiom;
import org.omwg.ontology.Ontology;
import org.wsmo.common.Entity;
import org.wsmo.common.TopEntity;
import org.wsmo.service.Capability;
import org.wsmo.service.ServiceDescription;
import org.wsmo.service.WebService;

public class WsmoTopEntityImport
{
	private List<LEShell> shells = new ArrayList<LEShell>();
	private List<Ontology> importedOntos = null;

	private String appendIdPart(String idPrefix, Entity entity, String defaultSuffix) {
		String suffix = Util.getLocalName(entity);
		if (suffix.length() == 0) {
			suffix = defaultSuffix;
		}
		if (idPrefix.length() == 0) {
			return suffix;
		}
		return idPrefix + "." + suffix;
	}

	private void addAxiomsLogicalExpressions(String idPrefix, Set axioms) {
		Iterator it = axioms.iterator();
		int i = 0; 
		while (it.hasNext()) {
			Axiom axiom = (Axiom)it.next();
			String defaultName = "axiom";
			if (i > 0) {
				defaultName += "_" + i;
			}
			String axiomId = appendIdPart(idPrefix, axiom, defaultName);
			String axiomName = appendIdPart("", axiom, defaultName);
			
			Iterator itLogExpr = axiom.listDefinitions().iterator();
			int j = 0;
			while (itLogExpr.hasNext()) {
				String leId = axiomId;
				if (j > 0) {
					leId += "_definition" + j;
				}
				LogicalExpression le = (LogicalExpression)itLogExpr.next();
				this.shells.add(new LEShell(axiomName, leId, le, this.importedOntos));
				++j;
			}
		}
	}

	private void addCapabilityLogicalExpressions(String idPrefix, Capability capability) {
		String id = appendIdPart(idPrefix, capability, "capability");

		addAxiomsLogicalExpressions(id + "_precondition", capability.listPreConditions());
		addAxiomsLogicalExpressions(id + "_assumption", capability.listAssumptions());
		addAxiomsLogicalExpressions(id + "_postcondition", capability.listPostConditions());
		addAxiomsLogicalExpressions(id + "_effect", capability.listEffects());
	}

	private void addServiceLogicalExpressions(String idPrefix, ServiceDescription serviceDescr) {
		String defaultName = (serviceDescr instanceof WebService) ? "service" : "goal"; 
		String id = appendIdPart(idPrefix, serviceDescr, defaultName);
		
		if (serviceDescr.getCapability() != null) {
			addCapabilityLogicalExpressions(id, serviceDescr.getCapability());
		}
	}

	private void addOntologyLogicalExpressions(String idPrefix, Ontology ontology) {
		String id = appendIdPart(idPrefix, ontology, "ontology");
		addAxiomsLogicalExpressions(id, ontology.listAxioms());
	}

	public List<LEShell> listLEShells(TopEntity[] tops) {
		this.shells.clear();
		if (tops == null) {
			return this.shells;
		}
		
		for (int i = 0; i < tops.length; i++) {
			this.importedOntos = Util.listImportedOntos(tops[i]);
			if (tops[i] instanceof ServiceDescription) {
				// This case handles both WebService and Goal.
				addServiceLogicalExpressions("", (ServiceDescription)tops[i]);
			}
			else if (tops[i] instanceof Capability) {
				addCapabilityLogicalExpressions("", (Capability)tops[i]);
			}
			else if (tops[i] instanceof Ontology) {
				addOntologyLogicalExpressions("", (Ontology)tops[i]);
			}
			else {
				// Other kinds of TopEntities do not contain logical expressions.
			}
		}
		return this.shells;
	}

	/**
	 * Displays a selection dialog containing the identifiers of all logical expressions from leShells. 
	 * @return The subset of the selected expressions. Null if the user pressed cancel.
	 */
	public List selectLogicalExpressions(List<LEShell> leShells) {
		LESelectionDialog dialog = new LESelectionDialog(null, leShells);
		if (dialog.open() == Window.OK) {
			return dialog.listSelected();
		}
		return null;
	}

}

