package org.infrawebs.axiom.model;

import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.io.Serializable;

import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertySource;

public abstract class AbstractModel implements IPropertySource, Serializable
{
	public static final String ID_PROPERTY_CONSTRAINT = "ID_PROPERTY_CONSTRAINT";

	private static final long serialVersionUID = 200510301500L;

	private Object constraint = null;
	// NOTE: The listeners are not serialized and must be restored manually. 
	private transient PropertyChangeSupport listeners = new PropertyChangeSupport(this);
	// The property descriptors need not be serialized. They are created on demand.
	private transient IPropertyDescriptor[] propertyDescriptors = null;

	public boolean hasListeners() {
		return this.listeners.hasListeners("");
	}
	
	public void addPropertyChangeListener(PropertyChangeListener listener) {
		this.listeners.addPropertyChangeListener(listener);
	}

	public void removePropertyChangeListener(PropertyChangeListener listener) {
		this.listeners.removePropertyChangeListener(listener);
	}

	public void firePropertyChange(String propName, Object oldValue, Object newValue) {
		this.listeners.firePropertyChange(propName, oldValue, newValue);
	}

	public Object getConstraint() {
		return this.constraint;
	}

	public void setConstraint(Object constraint) {
		this.constraint = constraint;
		firePropertyChange(ID_PROPERTY_CONSTRAINT, null, constraint);
	}

	public Object getEditableValue() {
		return this;
	}

	public IPropertyDescriptor[] getPropertyDescriptors() {
		if (this.propertyDescriptors == null) {
			this.propertyDescriptors = createPropertyDescriptors();
		}
		if (this.propertyDescriptors == null) {
			this.propertyDescriptors = new IPropertyDescriptor[0];
		}
		return this.propertyDescriptors;
	}

	public abstract IPropertyDescriptor[] createPropertyDescriptors();

	public abstract Object getPropertyValue(Object id);

	public abstract void setPropertyValue(Object id, Object value);

	public boolean isPropertySet(Object id) {
		return true;
	}

	public void resetPropertyValue(Object id) {
		// By default do nothing. 
	}

	private void readObject(java.io.ObjectInputStream in)
			throws IOException, ClassNotFoundException {
		in.defaultReadObject();
		// NOTE: Subclasses must restore all required listeners manually.
		this.listeners = new PropertyChangeSupport(this);
	}
	
}
