package org.infrawebs.axiom.model;

import org.eclipse.draw2d.geometry.Point;
import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.gef.requests.CreationFactory;
import org.infrawebs.axiom.plugin.AxiomEditorPlugin;
import org.infrawebs.common.Messages;
import org.infrawebs.common.Util;
import org.infrawebs.ontology.shells.ParameterShell;
import org.omwg.ontology.Concept;
import org.omwg.ontology.Instance;
import org.omwg.ontology.OntologyElement;
import org.omwg.ontology.Relation;
import org.wsmo.common.exception.SynchronisationException;

public class ModelFactory implements CreationFactory {
	
	public class ImportedElementException extends Exception
	{
		private static final long serialVersionUID = 200509271500L;

		public ImportedElementException(String message) {
			super(message);
		}
	}

	private AxiomContentsModel contents = null;
	private OntologyElement seed = null;

	public ModelFactory(AxiomContentsModel contents) {
		this.contents = contents;
	}

	public void setSeed(OntologyElement seed) {
		this.seed = seed;
	}

	public Object getObjectType() {
		return null;
	}

	public Object getNewObject() {
		try {
			if (this.seed instanceof Concept) {
				Concept concept = (Concept)this.seed;
				if (Util.isValueType(concept))
					return createValueInstance(concept);
				return createVariable(concept);
			}
			else if (this.seed instanceof Instance) {
				Instance instance = (Instance)this.seed;
				// TODO: Find the exact concept which the user expanded in the ontology.
				Concept parent = (Concept)instance.listConcepts().iterator().next();
				return createInstance(instance, parent);
			}
			else if (this.seed instanceof Relation) {
				return createRelation((Relation)this.seed); 
			}
			else {
				Messages.showWarning("Cannot add this object directly to the axiom.");
			}
		}
		catch (ImportedElementException ex) {
			Messages.showWarning(ex.getMessage());
		}
		return null;
	}

	private VariableModel createVariable(Concept concept) throws ImportedElementException {
		Concept definition = (Concept)AxiomEditorPlugin.ontoStore.findDefinition(concept);
		if (definition == null) {
			// Try to load the ontology in which this concept is defined.
			definition = (Concept)AxiomEditorPlugin.ontoLoader.jumpToDefinition(concept);
			if (definition == null) {
				throw new ImportedElementException(
						"Cannot find the definition of this concept. The ontology in which it is defined is missing.");
			}
		}
		
		return new VariableModel(this.contents, definition);
	}

	private RelationModel createRelation(Relation relation) throws ImportedElementException {
		relation = (Relation)AxiomEditorPlugin.ontoStore.findDefinition(relation);
		if (relation == null) {
			throw new ImportedElementException("Cannot use relations defined in imported ontologies.");
		}

		return new RelationModel(this.contents, relation);
	}

	private InstanceModel createInstance(Instance instance, Concept parent)
			throws SynchronisationException, ImportedElementException {
		instance = (Instance)AxiomEditorPlugin.ontoStore.findDefinition(instance);
		if (instance == null) {
			throw new ImportedElementException("Cannot use instances defined in imported ontologies.");
		}

		InstanceModel model = new InstanceModel(this.contents, instance, parent);
		return model;
	}

	private ValueInstanceModel createValueInstance(Concept concept)
			throws SynchronisationException, ImportedElementException {
		concept = (Concept)AxiomEditorPlugin.ontoStore.findDefinition(concept);
		if (concept == null) {
			throw new ImportedElementException(
					"Cannot use instances whose types are defined in imported ontologies.");
		}

		ValueInstanceModel model = new ValueInstanceModel(this.contents, concept);
		return model;
	}

	public ModelWithConnections createAxiomElement(OntologyElement forSeed) {
		setSeed(forSeed);
		ModelWithConnections model = (ModelWithConnections)getNewObject();
		if (model == null) {
			return null;
		}
		Point location = Util.getRandomLocation();
		model.setConstraint(new Rectangle(location.x, location.y, -1, -1));
		return model;
	}

	public RelationModel createRelation(ParameterShell paramInfo) {
		Relation relation = paramInfo.getParameter().getRelation();
		return (RelationModel)createAxiomElement(relation);
	}

	public OperatorModel createOperator(int operatorType, Point location) {
		OperatorModel model = new OperatorModel(this.contents, operatorType);
		if (location == null) {
			location = Util.getRandomLocation();
		}
		model.setConstraint(new Rectangle(location.x, location.y, -1, -1));
		return model;
	}


}
