package org.infrawebs.axiom.model;

import java.util.List;

import org.eclipse.draw2d.geometry.Rectangle;
import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.infrawebs.common.Messages;

public class OperatorModel extends ModelWithConnections
{
	private static final long serialVersionUID = 200512242100L;

	public static final String ID_PROPERTY_OPERATOR_TYPE = "ID_PROPERTY_OPERATOR_TYPE";
	private static final String[] OPERATORS = {"OR", "AND", "NOT"};
	public static final int OPERATOR_OR = 0;
	public static final int OPERATOR_AND = 1;
	public static final int OPERATOR_NOT = 2;
	private int operatorType = OPERATOR_OR;
	
	public OperatorModel(AxiomContentsModel contents, int operatorType) {
		super(contents);
		setOperatorType(operatorType);
	}
	
	public String getOperatorTypeAsString() {
		return OPERATORS[this.operatorType];
	}
	
	public static String getOperatorTypeAsString(int operatorType) {
		return OPERATORS[operatorType];
	}
	
	public int getOperatorType() {
		return this.operatorType;
	}
	
	public void setOperatorType(int operatorType) {
		if (operatorType >= 0 && operatorType < OPERATORS.length) {
			this.operatorType = operatorType;
			firePropertyChange(ID_PROPERTY_OPERATOR_TYPE, null, new Integer(operatorType));
		}
		else
			Messages.showError("Invalid operator type: " + operatorType);
	}

	@Override
	public IPropertyDescriptor[] createPropertyDescriptors() {
		IPropertyDescriptor[] descriptors =	new IPropertyDescriptor[1];
		descriptors[0] = new ComboBoxPropertyDescriptor(ID_PROPERTY_OPERATOR_TYPE, "Operator type", OPERATORS);
		return descriptors;
	}

	@Override
	public Object getPropertyValue(Object id) {
		if (id.equals(ID_PROPERTY_OPERATOR_TYPE)) {
			return new Integer(this.operatorType);
		}
		return null;
	}

	@Override
	public void setPropertyValue(Object id, Object value) {
		if (id.equals(ID_PROPERTY_OPERATOR_TYPE)) {
			setOperatorType(((Integer)value).intValue());
		}
	}
	
	// An OperatorModel should have exactly 1 parent. 
	public ModelWithConnections getParent() {
		List<ModelWithConnections> parents = listGrafParents();
		if (parents.size() > 0) {
			return parents.get(0); 
		}
		return null;
	}

	public List<ModelWithConnections> listOperands() {
		return listGrafChildren();
	}
	
	public boolean hasEnoughOperands() {
		return	(this.operatorType == OPERATOR_OR && listOperands().size() >= 2) ||
				(this.operatorType == OPERATOR_AND && listOperands().size() >= 2) ||
				(this.operatorType == OPERATOR_NOT && listOperands().size() == 1);
	}

	@Override
	public void setConstraint(Object constraint) {
		Rectangle newRect = (Rectangle)constraint;
		// Keep models auto sized.
		newRect.setSize(-1, -1);			
		super.setConstraint(newRect);
	}
	
}
