package org.infrawebs.axiom.model;

import java.io.IOException;

import org.infrawebs.axiom.plugin.AxiomEditorPlugin;
import org.infrawebs.common.Util;
import org.infrawebs.ontology.RestorationException;
import org.infrawebs.ontology.shells.ParameterShell;
import org.omwg.ontology.Relation;
import org.wsmo.common.Identifier;

public class ParameterModel extends RefinableModel
{
	private static final long serialVersionUID = 200512242100L;

	public ParameterModel(AxiomContentsModel contents, ParameterShell paramInfo) {
		super(contents, paramInfo);
	}

	/**
	 * The default variable name is based on the parameter's type.
	 */
	@Override
	public VarNameModel createVarNameModel() {
		String prefix = Util.getLocalName(this.refinable.getFirstType());
		return new VarNameModel(this.contents.getVarNameList(), prefix);
	}
	
	private void writeObject(java.io.ObjectOutputStream out) throws IOException {
		out.defaultWriteObject();
		// Do not write the parameter shell itself. Instead, write all it's fields manually.
		// Replace ontology objects with their identifiers.
		ParameterShell paramShell = (ParameterShell)getRefinable(); 
		out.writeObject(paramShell.getParameter().getRelation().getIdentifier());
		out.writeInt(paramShell.getPosition());
	}

	private void readObject(java.io.ObjectInputStream in)
			throws IOException, ClassNotFoundException {
		in.defaultReadObject();
		// Read the elements that were saved for the parameter shell.
		Identifier relationId = (Identifier)in.readObject();
		int cachedPosition = in.readInt();

		// Restore the definition of the parameter's relation.
		Relation relation = (Relation)AxiomEditorPlugin.ontoStore.findDefinition(
				relationId, Relation.class);
		if (relation == null) {
			throw new RestorationException("Relation " + relationId + " not found\n" +
					"while loading parameter " + cachedPosition + ".");
		}
		
		// Use the restored relation and cachedPosition to find the parameter object.
		if (cachedPosition > 0 && cachedPosition <= relation.listParameters().size()) {
			byte index = (byte)(cachedPosition - 1);
			this.refinable = new ParameterShell(relation.getParameter(index));
		}
		else {
			throw new RestorationException("Invalid parameter position " +
					cachedPosition + " in relation " + relationId + ".");
		}
	}

}
