package org.infrawebs.axiom.model;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;

import org.eclipse.ui.views.properties.ComboBoxPropertyDescriptor;
import org.eclipse.ui.views.properties.IPropertyDescriptor;
import org.eclipse.ui.views.properties.TextPropertyDescriptor;
import org.infrawebs.common.Messages;

public abstract class VarNameHolderModel extends ModelWithConnections
		implements PropertyChangeListener
{
	public static final String ID_PROPERTY_VARNAME_MODEL = "ID_PROPERTY_VARNAME_MODEL";

	private static final long serialVersionUID = 200512242100L;

	private VarNameModel varNameModel = null;

	public VarNameHolderModel(AxiomContentsModel contents) {
		super(contents);
	}

	public VarNameModel getVarNameModel() {
		return this.varNameModel;
	}
	
	public void setVarNameModel(VarNameModel varNameModel) {
		VarNameModel oldVarNameModel = this.varNameModel;
		this.varNameModel = varNameModel;
		
		// Receive notifications when the varNameModel changes its name.
		if (oldVarNameModel != null) {
			oldVarNameModel.removePropertyChangeListener(this);
		}
		if (varNameModel != null) {
			varNameModel.addPropertyChangeListener(this);
		}
		
		firePropertyChange(ID_PROPERTY_VARNAME_MODEL, oldVarNameModel, varNameModel);
	}
	
	public void propertyChange(PropertyChangeEvent evt) {
		// Resend the change in the varNameModel's name. 
		if (evt.getPropertyName().equals(VarNameModel.ID_PROPERTY_VARNAME) ||
				evt.getPropertyName().equals(VarNameModel.ID_PROPERTY_EXISTS_FLAG)) {
			firePropertyChange(evt.getPropertyName(), evt.getOldValue(), evt.getNewValue());
		}
	}
	
	@Override
	public IPropertyDescriptor[] createPropertyDescriptors() {
		IPropertyDescriptor[] descriptors = new IPropertyDescriptor[2];
		descriptors[0] = new TextPropertyDescriptor(VarNameModel.ID_PROPERTY_VARNAME, "Variable");
		descriptors[1] = new ComboBoxPropertyDescriptor(VarNameModel.ID_PROPERTY_EXISTS_FLAG, "Exists", new String[] {"false", "true"});
		return descriptors;
	}

	@Override
	public Object getPropertyValue(Object id) {
		// Handle events addressed to the varname model.
		if (id.equals(VarNameModel.ID_PROPERTY_VARNAME)) {
			return (this.varNameModel != null)
					? this.varNameModel.getPropertyValue(id) : "";
		}
		else if	(id.equals(VarNameModel.ID_PROPERTY_EXISTS_FLAG)) {
			return (this.varNameModel != null)
					? this.varNameModel.getPropertyValue(id) : new Integer(0);
		}
		return null;
	}

	@Override
	public void setPropertyValue(Object id, Object value) {
		if (id.equals(VarNameModel.ID_PROPERTY_VARNAME)) {
			if (this.varNameModel != null) {
				this.varNameModel.setPropertyValue(id, value);
			}
			else {
				Messages.showError("BUG? VarNameModel missing!");
			}
		}
		else if (id.equals(VarNameModel.ID_PROPERTY_EXISTS_FLAG)) {
			if (this.varNameModel != null) {
				this.varNameModel.setPropertyValue(id, value);
			}
			else {
				Messages.showWarning("Variable not initialized.");
			}
		}
	}

	private void readObject(java.io.ObjectInputStream in)
			throws IOException, ClassNotFoundException {
		in.defaultReadObject();
		// Since the list of listeners is not persisted, register this object as a listener manually.
		if (this.varNameModel != null) {
			this.varNameModel.addPropertyChangeListener(this);
		}
	}

}
