package org.infrawebs.axiom.model;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;

import org.infrawebs.axiom.model.VarNameModel.VarNameException;

public class VarNameList implements Serializable
{
	private static final long serialVersionUID = 200509271500L;

	private HashMap<String, VarNameModel> varNameList = new HashMap<String, VarNameModel>();

	// Adds a VarNameModel with a unique variable name to the list.
	public void add(VarNameModel varNameModel) throws VarNameException {
		// Do nothing if the model is already registered.
		if (this.varNameList.values().contains(varNameModel)) {
			return;
		}

		// Prevent adding models with duplicate names.
		String varName = varNameModel.getName();
		if (this.varNameList.get(varName) != null) {
			throw new VarNameException("Cannot register variable: duplicate name \""
					+ varName + "\".");
		}
		
		this.varNameList.put(varName, varNameModel);
	}

	public void remove(String varName) {
		this.varNameList.remove(varName);
	}

	public VarNameModel get(String varName) {
		return this.varNameList.get(varName);
	}

	// Appends the smallest possible number to the prefix so that
	// name duplication is avoided. 
	public String generateUniqueVariableName(String variablePrefix) {
		if (variablePrefix.length() == 0) {
			variablePrefix = "variable";
		}
		
		if (!this.varNameList.containsKey(variablePrefix)) {
			return variablePrefix;
		}
		
		int num = 1;
		while (this.varNameList.containsKey(variablePrefix + num)) {
			num++;
		}
		return variablePrefix + num;
	}
	
	public Iterator iterator() {
		return this.varNameList.values().iterator();
	}

}
