package org.infrawebs.axiom.model.commands;


import org.eclipse.gef.commands.Command;
import org.infrawebs.axiom.model.ConnectionModel;
import org.infrawebs.axiom.model.ModelWithConnections;

public class ConnectionReconnectCommand extends Command {
	private ConnectionModel connection;
	private ModelWithConnections newSource = null;
	private ModelWithConnections newTarget = null;
	private ModelWithConnections oldSource = null;
	private ModelWithConnections oldTarget = null;

	public ConnectionReconnectCommand(ConnectionModel connection) {
		this.connection = connection;
	}
	
	@Override
	public void execute() {
		if (this.newSource != null) {
			this.oldSource = this.connection.getSource();
			reconnectSource(this.newSource);
		}

		if (this.newTarget != null) {
			this.oldTarget = this.connection.getTarget();
			reconnectTarget(this.newTarget);
		}
	}

	private void reconnectSource(ModelWithConnections source) {
		this.connection.detachSource();
		this.connection.setSource(source);
		this.connection.attachSource();
	}

	private void reconnectTarget(ModelWithConnections target) {
		this.connection.detachTarget();
		this.connection.setTarget(target);
		this.connection.attachTarget();
//		
//		// Notify the connection Source that its children have changed, forcing the Outline to refresh. 
//		connection.getSource().firePropertyChange("neshto", null, null);
	}
	
	public ConnectionModel getConnection() {
		return this.connection;
	}

	public void setNewSource(Object model) {
		this.newSource = (ModelWithConnections) model;
	}

	public void setNewTarget(Object model) {
		this.newTarget = (ModelWithConnections) model;
	}

	@Override
	public void undo() {
		if (this.oldSource != null)
			reconnectSource(this.oldSource);
		if (this.oldTarget != null)
			reconnectTarget(this.oldTarget);

		this.oldSource = null;
		this.oldTarget = null;
	}
}
