package org.infrawebs.axiom.outline;

import java.util.Observable;
import java.util.Observer;

import org.eclipse.draw2d.LightweightSystem;
import org.eclipse.draw2d.Viewport;
import org.eclipse.draw2d.parts.ScrollableThumbnail;
import org.eclipse.gef.ContextMenuProvider;
import org.eclipse.gef.EditDomain;
import org.eclipse.gef.GraphicalViewer;
import org.eclipse.gef.LayerConstants;
import org.eclipse.gef.editparts.ScalableRootEditPart;
import org.eclipse.gef.ui.actions.ActionRegistry;
import org.eclipse.gef.ui.parts.ContentOutlinePage;
import org.eclipse.gef.ui.parts.SelectionSynchronizer;
import org.eclipse.gef.ui.parts.TreeViewer;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.SashForm;
import org.eclipse.swt.events.DisposeEvent;
import org.eclipse.swt.events.DisposeListener;
import org.eclipse.swt.widgets.Canvas;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Tree;
import org.eclipse.ui.IActionBars;
import org.eclipse.ui.actions.ActionFactory;
import org.eclipse.ui.part.IPageSite;
import org.infrawebs.axiom.editor.AxiomEditor;
import org.infrawebs.axiom.editor.ContextMenuProviderImpl;
import org.infrawebs.axiom.outline.editparts.TreeEditPartFactory;
import org.infrawebs.axiom.plugin.AxiomEditorPlugin;
import org.infrawebs.common.Util;

public class AxiomTreeOutlinePage extends ContentOutlinePage implements Observer {

	private AxiomEditor editor = null;
	private SashForm sash = null;
	private DisposeListener disposeListener = null;
	private ActionRegistry registry;
	private TreeViewer treeViewer = null;
	ScrollableThumbnail thumbnail = null;
	
	public AxiomTreeOutlinePage(ActionRegistry registry) {
		super(new TreeViewer());
		// get the same TreeViewer that we supplied to the super constructor
		this.treeViewer = (TreeViewer)this.getViewer();
		
		this.registry = registry;
	}
	
	@Override
	public void init(IPageSite pageSite) {
		super.init(pageSite);
		this.editor = AxiomEditorPlugin.getAxiomEditor();
		
		ActionRegistry editorRegistry = (ActionRegistry)this.editor.getAdapter(ActionRegistry.class); 
		IActionBars bars = pageSite.getActionBars();

		String id = ActionFactory.UNDO.getId();
		bars.setGlobalActionHandler(id, editorRegistry.getAction(id));

		id = ActionFactory.REDO.getId();
		bars.setGlobalActionHandler(id, editorRegistry.getAction(id));

		id = ActionFactory.DELETE.getId();
		bars.setGlobalActionHandler(id, editorRegistry.getAction(id));

//		Example: how to add action to the view		
//		id = ActionFactory.DELETE.getId();
//		bars.getToolBarManager().add(registry.getAction(id));
		
		bars.updateActionBars();
	}

	@Override
	public void createControl(Composite parent) {
//		this.sash = new SashForm(parent, SWT.VERTICAL);
        this.sash = new SashForm(parent, SWT.HORIZONTAL);
		this.treeViewer.createControl(this.sash);
		EditDomain editDomain = (EditDomain)this.editor.getAdapter(EditDomain.class); 
		this.treeViewer.setEditDomain(editDomain);
		this.treeViewer.setEditPartFactory(new TreeEditPartFactory());
		this.treeViewer.setContents(this.editor.getContents().getAxiomContentsModel());
		SelectionSynchronizer synchronizer = (SelectionSynchronizer)this.editor.getAdapter(SelectionSynchronizer.class);
		synchronizer.addViewer(this.treeViewer);
		// The editor notifies the Outline on command stack changes.
		this.editor.getContents().addObserver(this);
		expandTree();
		
		Canvas canvas = new Canvas(this.sash, SWT.BORDER);
		LightweightSystem lws = new LightweightSystem(canvas);

		GraphicalViewer viewer = (GraphicalViewer)this.editor.getAdapter(GraphicalViewer.class); 
		ScalableRootEditPart root = (ScalableRootEditPart)viewer.getRootEditPart(); 
		this.thumbnail = new ScrollableThumbnail((Viewport)root.getFigure());
		this.thumbnail.setSource(root.getLayer(LayerConstants.PRINTABLE_LAYERS));

		lws.setContents(this.thumbnail);

		this.disposeListener = new DisposeListener() {
			public void widgetDisposed(DisposeEvent e) {
				if (AxiomTreeOutlinePage.this.thumbnail != null) {
					AxiomTreeOutlinePage.this.thumbnail.deactivate();
					AxiomTreeOutlinePage.this.thumbnail = null;
				}
			}
		};
		viewer.getControl().addDisposeListener(this.disposeListener);

		ContextMenuProvider menuManager = new ContextMenuProviderImpl(this.registry, viewer);
		this.treeViewer.setContextMenu(menuManager);
	}

	@Override
	public Control getControl() {
		return this.sash;
	}

	@Override
	public void dispose() {
		SelectionSynchronizer synchronizer = (SelectionSynchronizer)this.editor.getAdapter(SelectionSynchronizer.class);
		synchronizer.removeViewer(this.treeViewer);
		// The editor notifies the Outline on command stack changes.
		this.editor.getContents().deleteObserver(this);

		GraphicalViewer viewer = (GraphicalViewer)this.editor.getAdapter(GraphicalViewer.class); 
		Control control = viewer.getControl();
		if (control != null && !control.isDisposed()) {
			control.removeDisposeListener(this.disposeListener);
		}

		super.dispose();
	}
	
	private void expandTree() {
		Tree outlineTree = (Tree)this.treeViewer.getControl(); 
		if (outlineTree.getItems().length > 0) {
			Util.ExpandFullyTreeItem(outlineTree.getItems()[0], true);
		}
	}

	public void update(Observable o, Object arg) {
		// Force a full refresh of the outline tree no matter what has changed.
		this.treeViewer.setContents(this.treeViewer.getContents());
		
		expandTree();
		// TODO: Restore selected tree item.
	}

}
