package org.infrawebs.axiom.plugin;

import java.net.URL;
import java.util.MissingResourceException;
import java.util.ResourceBundle;

import org.eclipse.core.runtime.Platform;
import org.eclipse.ui.IEditorPart;
import org.eclipse.ui.IWorkbenchPage;
import org.eclipse.ui.plugin.AbstractUIPlugin;
import org.infrawebs.axiom.editor.AxiomEditor;
import org.infrawebs.axiom.model.AxiomContentsModel;
import org.infrawebs.common.Messages;
import org.infrawebs.ontology.OntoLoader;
import org.infrawebs.ontology.OntoStore;
import org.osgi.framework.BundleContext;

/**
 * The main plugin class to be used in the desktop.
 */
public class AxiomEditorPlugin extends AbstractUIPlugin
{
	// The shared instance of the plugin.
	private static AxiomEditorPlugin plugin;

	private static final String BUNDLE_NAME = "org.infrawebs.axiom.plugin.resource"; //$NON-NLS-1$
	private static final ResourceBundle resourceBundle = ResourceBundle.getBundle(BUNDLE_NAME);

	// Global ontology management.
	public static final OntoStore ontoStore = new OntoStore();
	public static final OntoLoader ontoLoader = new OntoLoader(ontoStore);

	public AxiomEditorPlugin() {
		super();
		plugin = this;
	}

	/**
	 * This method is called upon plug-in activation.
	 */
	@Override
	public void start(BundleContext context) throws Exception {
		super.start(context);
		loadServiceOntologies();
	}

	/**
	 * This method is called when the plug-in is stopped.
	 */
	@Override
	public void stop(BundleContext context) throws Exception {
		super.stop(context);
	}

	private void loadServiceOntologies() {
		URL fileURL = getBundle().getEntry("/service/service.wsml");
		try {
			String fileName = Platform.asLocalURL(fileURL).getFile();
			AxiomEditorPlugin.ontoLoader.loadOntologiesFromFile(fileName, true);
		}
		catch (Exception ex) {
			Messages.showError("Error loading service ontologies.\n" + ex.getMessage());
		}
	}		

	/**
	 * Returns the shared instance.
	 */
	public static AxiomEditorPlugin getDefault() {
		return plugin;
	}

	/**
	 * Private writable space within the .metadata directory.
	 */ 
	public String getServicePath() {
		return getDefault().getStateLocation().toString();
	}

	/**
	 * Returns the string from the plugin's resource bundle,
	 * or 'key' if not found.
	 */
	public static String getResourceString(String key) {
		try {
			return (resourceBundle != null) ? resourceBundle.getString(key) : key;
		} catch (MissingResourceException e) {
			return key;
		}
	}

	public static AxiomEditor getAxiomEditor() {
		IWorkbenchPage page = getDefault().getWorkbench().getActiveWorkbenchWindow().getActivePage();
		if (page == null) {
			return null;
		}
		IEditorPart editorPart = page.getActiveEditor();
		return (editorPart instanceof AxiomEditor) ? (AxiomEditor)editorPart : null;
	}

	public static AxiomContentsModel getAxiomContentsModel() {
		AxiomEditor editor = getAxiomEditor();
		return (editor != null) ? editor.getContents().getAxiomContentsModel() : null;
	}
}
