package org.infrawebs.axiom.text.actions;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStreamWriter;

import org.eclipse.swt.SWT;
import org.eclipse.swt.graphics.Image;
import org.eclipse.swt.widgets.FileDialog;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.IWorkbenchPart;
import org.infrawebs.axiom.editor.AxiomEditor;
import org.infrawebs.axiom.editor.actions.AxiomAction;
import org.infrawebs.axiom.plugin.AxiomEditorPlugin;
import org.infrawebs.axiom.text.AxiomTextView;
import org.infrawebs.common.Messages;
import org.infrawebs.ontology.ui.OntoImages;

public class SaveAxiomTextAction extends AxiomAction
{
	private AxiomTextView view = null;
	
	public SaveAxiomTextAction(IWorkbenchPart part) {
		super(part);
		this.view = (AxiomTextView)part;
	}

	@Override
	public void init() { 
		super.init();
		setToolTipText("Save axiom text as a wsml file.");
	}
	
	@Override
	protected String getCaption() {
		return "Save axiom text";
	}

	@Override
	protected Image getImage() {
		return OntoImages.getImage(OntoImages.SAVE_FILE);
	}

	@Override
	public boolean calculateVisible() {
		return true;
	}

	@Override
	public void run() {
		AxiomEditor editor = AxiomEditorPlugin.getAxiomEditor();
		if (editor == null) {
			return;
		}

		// Display a "Save as..." dialog.
		Shell shell = AxiomEditorPlugin.getDefault().getWorkbench().
				getActiveWorkbenchWindow().getShell();
		FileDialog dialog = new FileDialog(shell, SWT.SAVE);
		dialog.setText("Save axiom text..."); //$NON-NLS-1$
		dialog.setFilterNames(new String[] {
				"Ontologies (*.wsml)",
				"All files (*.*)"
		});
		dialog.setFilterExtensions(new String[] {
				"*.wsml", //$NON-NLS-1$
				"*.*" //$NON-NLS-1$
		});
		String fileName = dialog.open();
		if (fileName == null || fileName.length() == 0) {
			return;
		}
		if (new File(fileName).exists() && false == Messages.showQuestion(
				fileName + " already exists.\nDo you want to replace it?")) {
			return;
		}

		// Get the text straight from the text control in the view.
		AxiomTextView.OutlinePage page = (AxiomTextView.OutlinePage)this.view.getCurrentPage();
		String text = page.getText();
		try {
			OutputStreamWriter os = new OutputStreamWriter(new BufferedOutputStream(
					new FileOutputStream(fileName)));
			try {
				os.write(text);
			}
			finally {
				os.close();
			}
		}
		catch (IOException ex) {
			Messages.showError("Error saving WSML text: " + ex.getMessage());
		}
	}

}
