package org.infrawebs.ontology;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.URL;

import org.infrawebs.common.Messages;

public class InternetLoader
{
	private OntoLoader ontoLoader = null;

	public InternetLoader(OntoLoader ontoLoader) {
		this.ontoLoader = ontoLoader;
	}

	private InputStream getInternetStream(String pathHint) {
		// Fill an array with propable URLs.
		String[] paths = new String[] {
				pathHint,
				pathHint + ".wsml"
		};
		
		for (int i = 0; i < paths.length; i++) {
			try {
				URL url = new URL(paths[i]);
				return url.openStream();
			}
			catch (IOException ex) {
				if (i == paths.length - 1) {
					Messages.showInformation("Ontology \"" + paths[i] +
							"\"\nwas not found on the Internet.");
				}
			}
		}
		return null;
	}

	private File downloadTempFile(InputStream is) {
		final int chunkSize = 8192;
		File tempFile = null;
		try {
			tempFile = File.createTempFile("onto", ".wsml");
			FileOutputStream os = new FileOutputStream(tempFile, false);
			try {
				byte[] data = new byte[chunkSize];
				int readBytes = is.read(data);
				while (readBytes >= 0) {
					os.write(data, 0, readBytes);
					readBytes = is.read(data);
				}
			}
			finally {
				os.close();
			}
			return tempFile;
		}
		catch (IOException ex) {
			Messages.showError("Error downloading ontology\n\n" +
					ex.getClass().getName() +
					"\nMessage: " + ex.getMessage() +
					"\nCause: " + ex.getCause());
			return null;
		}
	}

	public boolean loadOntologies(String urlHint) {
		InputStream is = getInternetStream(urlHint);
		if (is == null) {
			return false;
		}
		File tempFile = downloadTempFile(is);
		if (tempFile == null) {
			return false;
		}
		try {
			boolean valid = this.ontoLoader.loadOntologiesFromFile(
					tempFile.getAbsolutePath(), false);
			if (valid) {
				addToWorkspace(tempFile);
			}
			return valid;
		}
		finally {
			tempFile.delete();
		}
	}

    public void addToWorkspace(File sourceFile) {
		File directory = new File("TODO: Get workspace firectory.");
		String wsmlFileName = directory.getPath() + File.separator + sourceFile.getName();
		File destFile = new File(wsmlFileName);
		try {
			copyFile(sourceFile, destFile);
			// TODO: Register in workspace.
		}
		catch (IOException ex) {
			System.out.println("Failed copying file to repository: " + sourceFile.getAbsolutePath());
		}
	}

	// Copies src file to dst file.
    // If the dst file does not exist, it is created
	void copyFile(File src, File dst) throws IOException {
		InputStream in = new FileInputStream(src);
		OutputStream out = new FileOutputStream(dst);
		try {
			// Transfer bytes from in to out
			byte[] buf = new byte[1024];
			int len;
			while ((len = in.read(buf)) > 0) {
				out.write(buf, 0, len);
			}
		}
		finally {
			in.close();
			out.close();
		}
	}

}
