package org.infrawebs.ontology.shells;

import java.util.Map;
import java.util.Set;

import org.infrawebs.ontology.ui.OntoImages;
import org.omwg.ontology.Attribute;
import org.omwg.ontology.Concept;
import org.omwg.ontology.Ontology;
import org.omwg.ontology.OntologyElement;
import org.omwg.ontology.Value;
import org.wsmo.common.IRI;
import org.wsmo.common.Identifier;
import org.wsmo.common.exception.InvalidModelException;
import org.wsmo.common.exception.SynchronisationException;

public class AttributeShell extends RefinableShell
{
	public static final int NEW = OntoImages.ATTRIBUTE_NEW;
	public static final int INHERITED = OntoImages.ATTRIBUTE_INHERITED;
	public static final int OVERRIDEN = OntoImages.ATTRIBUTE_OVERRIDEN;
	
	private Attribute attribute = null;
	private Concept concept = null;
	private int inheritanceType; 
	
	public AttributeShell(Attribute attribute, Concept concept, int type) {
		this.attribute = attribute;
		this.concept = concept;
		this.inheritanceType = type;
	}

	public Concept getConcept() {
		return this.concept;
	}

	public Attribute getAttribute() {
		return this.attribute;
	}

	public int getInheritanceType() {
		return this.inheritanceType;
	}

	public void setInheritanceType(int type) {
		this.inheritanceType = type;
	}

	public String getTypeAsString() {
		switch (this.inheritanceType) {
			case NEW: return "new";
			case INHERITED: return "inherited";
			case OVERRIDEN: return "overriden";
			default:
				return "<type not defined>";		
		}
	}
	
	public String getCardinalityAsString() {
		return this.attribute.getMinCardinality() + " - " + this.getMaxCardinalityAsString();
	}

	public String getMaxCardinalityAsString() {
		int max = this.attribute.getMaxCardinality();
		String maxStr = (max == Integer.MAX_VALUE) ? "*" : Integer.toString(max);
		return maxStr;
	}

	@Override
	public boolean equals(Object target) {
        if (target == this) {
            return true;  // instance match
        }
        if (!(target instanceof AttributeShell)) {
            return false;
        }
		AttributeShell attrInfo = (AttributeShell)target;
		return (this.attribute == attrInfo.attribute) &&
				(this.concept == attrInfo.concept) &&
				(this.inheritanceType == attrInfo.inheritanceType);
	}

	// RefinableShell methods.

	@Override
	protected Set internalListTypes() {
		return this.attribute.listTypes();		
	}

	@Override
	public boolean isConstraining() {
		return this.attribute.isConstraining();
	}

	@Override
	public OntologyElement getParent() {
		return this.attribute.getConcept();
	}
	
	// OntologyElement methods.
	
	public Identifier getIdentifier() {
		return this.attribute.getIdentifier();
	}
	public Set listNFPValues(IRI key) {
		return this.attribute.listNFPValues(key);
	}
	public Map listNFPValues() throws SynchronisationException {
		return this.attribute.listNFPValues();
	}
	public void addNFPValue(IRI key, Identifier value) throws SynchronisationException, InvalidModelException {
		this.attribute.addNFPValue(key, value);
	}
	public void addNFPValue(IRI key, Value value) throws SynchronisationException, InvalidModelException {
		this.attribute.addNFPValue(key, value);
	}
	public void removeNFPValue(IRI key, Identifier value) throws SynchronisationException, InvalidModelException {
		this.attribute.removeNFPValue(key, value);
	}
	public void removeNFPValue(IRI key, Value value) throws SynchronisationException, InvalidModelException {
		this.attribute.removeNFPValue(key, value);
	}
	public void removeNFP(IRI key) throws SynchronisationException, InvalidModelException {
		this.attribute.removeNFP(key);
	}
	public Ontology getOntology() throws SynchronisationException {
		return this.attribute.getConcept().getOntology();
	}
	public void setOntology(Ontology ontology) throws SynchronisationException, InvalidModelException {
		this.attribute.getConcept().setOntology(ontology);
	}

}
