package org.infrawebs.ontology.shells;

import java.util.*;

import org.omwg.ontology.*;
import org.wsmo.common.*;
import org.wsmo.common.exception.*;
import org.wsmostudio.runtime.WSMORuntime;

public class ParameterShell extends RefinableShell
{
	private Parameter parameter = null;
	// NOTE: The first parameter will have cachedPosition "1". "0" is an invalid cachedPosition.
	private byte cachedPosition = 0;
	
	public ParameterShell(Parameter parameter) {
		this.parameter = parameter;
		findPosition();
	}

	private void findPosition() {
		Relation relation = this.parameter.getRelation();
		int count = relation.listParameters().size();
		for (byte i = 0; i < count; i++) {
			if (this.parameter == relation.getParameter(i)) {
				this.cachedPosition = (byte)(i + 1);
				return;
			}
		}
	}
	
	public Parameter getParameter() {
		return this.parameter;
	}

	public int getPosition() {
		return this.cachedPosition;
	}

//	public byte getPosition() {
//		return this.position;
//	}

	@Override
	public boolean equals(Object target) {
        if (target == this) {
            return true;  // instance match
        }
        if (!(target instanceof ParameterShell)) {
            return false;
        }
       	ParameterShell paramInfo = (ParameterShell)target;
		return getIdentifier().equals(paramInfo.getIdentifier());
	}

	// RefinableShell methods.

	@Override
	protected Set internalListTypes() {
		return this.parameter.listTypes();		
	}

	@Override
	public boolean isConstraining() {
		return this.parameter.isConstraining();
	}

	@Override
	public OntologyElement getParent() {
		return this.parameter.getRelation();
	}
	
	// OntologyElement methods.
	
	public Identifier getIdentifier() {
		if (this.parameter == null) {
			return WSMORuntime.getRuntime().getWsmoFactory().createAnonymousID();
		}

		// Use the cachedPosition of this parameter among the parameters of its relation
		// and return an identifier such as "...#lessThan2".
		return WSMORuntime.getRuntime().getWsmoFactory().createIRI(
				this.parameter.getRelation().getIdentifier().toString() +
				this.cachedPosition);
	}
	public Set listNFPValues(IRI key) {
		return new HashSet();
	}
	public Map listNFPValues() throws SynchronisationException {
		return new HashMap();
	}
	public void addNFPValue(IRI key, Identifier value) throws SynchronisationException, InvalidModelException {
		throw new InvalidModelException();
	}
	public void addNFPValue(IRI key, Value value) throws SynchronisationException, InvalidModelException {
		throw new InvalidModelException();
	}
	public void removeNFPValue(IRI key, Identifier value) throws SynchronisationException, InvalidModelException {
		throw new InvalidModelException();
	}
	public void removeNFPValue(IRI key, Value value) throws SynchronisationException, InvalidModelException {
		throw new InvalidModelException();
	}
	public void removeNFP(IRI key) throws SynchronisationException, InvalidModelException {
		throw new InvalidModelException();
	}
	public Ontology getOntology() throws SynchronisationException {
		return (this.parameter != null) ? this.parameter.getRelation().getOntology() : null;
	}
	public void setOntology(Ontology ontology) throws SynchronisationException, InvalidModelException {
		if (this.parameter == null) {
			throw new InvalidModelException();
		}
		this.parameter.getRelation().setOntology(ontology);
	}

}
