package org.infrawebs.ontology.ui;

import org.eclipse.jface.window.Window;
import org.eclipse.swt.SWT;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Shell;
import org.eclipse.ui.dialogs.SelectionDialog;
import org.infrawebs.common.Messages;
import org.infrawebs.ontology.OntoFilter;
import org.infrawebs.ontology.OntoStore;
import org.omwg.ontology.OntologyElement;

public class OntoSelectionDialog extends SelectionDialog
{
	private OntoStore ontoStore = null;
	private OntoFilter ontoFilter = null;
	private OntoPanes ontoPanes = null;
	private OntologyElement ontologySelection = null;
	
	public OntoSelectionDialog(Shell parentShell, OntoStore ontoStore, OntoFilter ontoFilter) {
		super(parentShell);
		this.ontoStore = ontoStore;
		this.ontoFilter = ontoFilter;
		this.setShellStyle(this.getShellStyle() | SWT.RESIZE | SWT.MAX);
	}

	@Override
	protected Control createDialogArea(Composite parent) {
		Composite composite = (Composite)super.createDialogArea(parent);

		this.ontoPanes = new OntoPanes(composite);
		this.ontoPanes.setOntoFilter(this.ontoFilter);
		this.ontoPanes.setOntoStore(this.ontoStore);
		
		if (this.ontoPanes.isEmpty()) {
			Messages.showWarning("No compatible items are available.");
			this.setReturnCode(Window.CANCEL);
			// Return immediately after open(). The calling function must call close()! 
			this.setBlockOnOpen(false);
		}
		else {
			GridData gd = new GridData(GridData.FILL_BOTH);
			gd.widthHint = 600;
			gd.heightHint = 400;
			this.ontoPanes.setLayoutData(gd);
			// Block the calling function until the user selects something.
			this.setBlockOnOpen(true);
		}
		
		return composite;
	}
	
	@Override
	public boolean close() {
		this.ontoPanes.dispose();
		return super.close();
	}

	public OntologyElement getOntologySelection() {
		return this.ontologySelection;
	}

	@Override
	protected void okPressed() {
		Object item = this.ontoPanes.getSelection();
		if (item instanceof OntologyElement) {
			this.ontologySelection = (OntologyElement)item;
			if (this.ontoFilter.allowSelect(this.ontologySelection)) {
				super.okPressed();
				return;
			}
		}
		// The dialog's title should suggest the right type of items to be selected.
		Messages.showWarning("Invalid item selected: " + this.getShell().getText());
	}
	
}
