package org.infrawebs.ontology.ui;

import java.beans.*;
import java.util.*;

import org.eclipse.jface.action.IToolBarManager;
import org.eclipse.jface.viewers.*;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.ui.*;
import org.eclipse.ui.part.ViewPart;
import org.eclipse.ui.views.properties.IPropertySource;
import org.infrawebs.ontology.*;
import org.infrawebs.ontology.actions.*;
import org.infrawebs.ontology.shells.OntologyShell;
import org.omwg.ontology.OntologyElement;

public abstract class OntoView extends ViewPart implements PropertyChangeListener, ISelectionProvider
{
	private OntoFilter ontoFilter = new OntoFilter(true);
	private OntoPanes ontoPanes = null;
    
    private List<ISelectionChangedListener> listeners = new LinkedList<ISelectionChangedListener>();

	@Override
	public void init(IViewSite site) throws PartInitException {
		super.init(site);
		
		IToolBarManager toolbar = site.getActionBars().getToolBarManager();
		toolbar.add(new OpenOntologyAction(this));
		site.getActionBars().updateActionBars();		

		getOntoStore().addListener(this);
	}

	@Override
	public void createPartControl(Composite parent) {
		parent.setLayout(new FillLayout());
		this.ontoPanes = createOntoPanes(parent);
		this.ontoPanes.setOntoFilter(this.ontoFilter);
		this.ontoPanes.setOntoStore(getOntoStore());
        getViewSite().setSelectionProvider(this);
	}

	@Override
	public void setFocus() {
		this.ontoPanes.setFocus();
	}

	@Override
	public void dispose() {
		getOntoStore().removeListener(this);
		super.dispose();
	}

	public void propertyChange(PropertyChangeEvent event) {
		if (event.getPropertyName().equals(OntoStore.JUMP_TO_ELEMENT)) {
			OntologyElement element = (OntologyElement)event.getNewValue();
			this.ontoPanes.jumpToElement(element);
		}
		else if (event.getPropertyName().equals(OntoStore.ADD_ONTOLOGY)) {
			OntologyShell ontology = (OntologyShell)event.getNewValue();
			this.ontoPanes.createPane(ontology);
			// update all other concepts with new attributes from the new ontology.
			this.ontoPanes.updateAllConcepts();
		}
	}

	public abstract OntoLoader getOntoLoader();
	public abstract OntoStore getOntoStore();

	protected OntoPanes createOntoPanes(Composite parent) {
		return new OntoPanes(parent);
	}

	@Override
	public Object getAdapter(Class adapter) {
	    if (adapter.equals(IPropertySource.class)) {
	        return this.ontoPanes.getSelectionPropertiesSource();
	    }
	    return super.getAdapter(adapter);
	}

	public ISelection getSelection() {
	    return new StructuredSelection(this);
	}

	public void setSelection(ISelection selection) {
	   // graphPane.setFocus();
	}

	public void removeSelectionChangedListener(ISelectionChangedListener listener) {
		this.listeners.remove(listener);
	}

	public void addSelectionChangedListener(ISelectionChangedListener listener) {
	    if (this.listeners.indexOf(listener) == -1) {
	    	this.listeners.add(listener);
	    }
	}

}
